from __future__ import print_function
## from builtins import input
## from builtins import range
import numpy as np
import math
import re
import scipy.optimize
import FortranIO
import universal_pattern as UP
 
''''

Solar-like Light-curve Simulator (SLS)

Simulate stochastically-excited oscillations and associated stellar background (granulation and activity).

Two different types of oscillation spectrum can be simulated:
- gen_up : oscillation spectrum based on the Universal Pattern  by Mosser et al (2011, A&A, 525, L9).
   Mixed-modes can been included when the asymptotic period spacing is specified.

- gen_adipls: oscillation spectrum based on eigenfrequencies computed with ADIPLS 

For more details see the helps associated with the procedures gen_up() and gen_adipls()

Other procedures are:

- genslc():  from an individual simulation, generate a series of simuated ligth-curves and save them in a set of files (one per light-curve). Each simulated light curve corresponds to a given realization. 

- savelc() : save a given light-curve in a ASCII file

- test_gen_up() : perform a simple illustrative test assuming the Universal Pattern (call to  gen_up)

- test_gen_adipls() : perform a simple illustrative test  using a set of eigenfrequencies computed with ADIPLS  (call to gen_adipls)



Copyright (c) 2014, Reza Samadi, LESIA - Observatoire de Paris

This is a free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
 
This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this code.  If not, see <http://www.gnu.org/licenses/>.

'''

# numax reference value (Mosser et al 2013, SF2A)
numaxref= 3104.

# solar constants
nucsun = 5300.  # sun cutt-off freq.
numaxsun = 3100.
deltanusun = 135.
teffsun = 5777.
msun = 1.98919e33           # solar mass
rsun = 6.9599e10            # solar radius
gmsun = 1.32712438e26       # G Msun
logg_sun = math.log10(gmsun) - 2.*math.log10(rsun)
ggrav = gmsun/msun          # grav. constant G

    
class SLSError(Exception):
    def __init__(self, value):
             self.value = value
    def __str__(self):
        return repr(self.value) 

def rebin1d(array,n):
    nr=int(float(array.shape[0])/float(n))
    return (np.reshape(array,(n,nr))).sum(1)

def gaussenvelop(nu,A,nu0,width):
    '''
    Gassian envelop for the oscillation spectrum

    INPUTS
    nu: frequencies
    A: amplitude of the envelope
    nu0: position in frequency (numax)
    width: width of the envelope
    
    '''
    return A*np.exp(-((nu-nu0)**2)/(width**2/(4*math.log(2)))) 
        
def plf(nu,h,tau,alpha):
    '''
    Pseudo Lorentzian Function (plf) for the granulation component(s)

    INPUTS:
    nu:  frequencies
    h: height
    tau : characteristic time
    alpha : slope
    
    '''

    return h/(1. + (2.*math.pi*tau *nu)**alpha)

def eft_2plf(h1,tau1,alpha1,h2,tau2,alpha2):
    # Given two pseudo lorenztian functions (plf), compute the associated e-folding time (eft)
    N = 2**19
    Nf = N/2  + 1
    tau = min(tau1,tau2)
    sampling =  tau*1e-3
    dnu = 1./(sampling*N)
    nu = np.arange(Nf) * dnu

    spc = plf(nu,h1,tau1,alpha1) + plf(nu,h2,tau2,alpha2)
    acf = np.fft.irfft(spc,n=N)
    acf = acf[1:N//2]/acf[0]
    time = (np.arange(N/2-1)+1.)*sampling/tau
    eft = np.interp(math.exp(-1),acf[::-1],time[::-1])*tau

    return eft

def mpsd2rts(f,mpsd,seed=None,time_shift=0.):
    '''

    Simulate a random time-series from a mean power spectral density spectrum

    time, ts, ps, seed =  mpsd2rts(f,mpsd,seed=None)

    INPUTS:
    f: frequencies (Hz)
    mpsd : mean power spectral density (PSD) in /Hz. Single-sided PSD
    seed: seed of the Random Number Generator (RNG)
    time_shift : imposes a time shift (seconds), default: 0.

    OUTPUTS:
    time: time values
    ts: simulated time-series
    ps: simulated power spectrum [/Hz]
    seed:  adopted value of the seed (chosen by the Random Number Generator if seed is not specified in input)
    
    '''

    # dnu: frequency resolution in Hz
    if (f[0] ==0. ):
        dnu = f[1]
        n = f.size
        i0 = 0
    else:
        dnu = f[0]
        n = f.size + 1
        i0 = 1

    # sampling time [s]
    dt = 1./(2.*dnu*n)

    # 
    time = np.arange(2*n) * dt  + time_shift

    if( seed == None):
        # see is not specified, we choose a random value 
        seed = np.random.random_integers(0,high=2**30)    
    # initialization of the state of the RNG
    np.random.seed(seed)
    
    
    # real and imaginary part randomly distributed
    Rr = np.random.normal(loc=0.,scale=1./math.sqrt(2.),size=n-i0)
    Ri = np.random.normal(loc=0.,scale=1./math.sqrt(2.),size=n-i0)


    # fourier spectrum
    tf = np.zeros(n,dtype=np.complex)
    
    tf[i0:] = np.sqrt(mpsd) * (Rr + Ri*1j)  * np.sqrt(n/dt)
    if(time_shift !=0):
        fp = np.arange(n)*dnu # frequencies, Hz
        tf[i0:] *= np.exp(2.*math.pi*fp[i0:]*time_shift*1j)

    # symmetrise
    tf = np.hstack((
                    np.array([0]),
                    tf[1:],
                    np.array([0]),
                    tf[:0:-1].conj()))        


    # generate the time series [ppm]
    ts = np.real(np.fft.ifft(tf)) 

    # power spectrum in /Hz
    ps = np.abs(tf[i0:n])**2 *(dt/n)

    return time,ts,ps,seed

def power_ratio(l,beta):
    '''

    power_ratio (l,i)

    Returns the mode relative power amplitude for a given angle i with respect to the mode power amplitude at the angle i=0, 
    i.e. returns the quantity given by (see eg. Dziembowski, 1971, Toutain & Goutterbroze 1993) :
    Q^2_lm = (l-m)!/(l+m)! ( P_lm[cos(i)] )^2
    where i is the inclination angle (i=0 corresponds to the direction of the rotation axis, ie. theta=0).  
        
    INPUTS:
    l: mode angular degree
    i: inclination angle in degree (i=0 points toward the observer)
    
    '''

    def combi (n,r):
        '''
        Compute combinations n r,  for  n>r

        '''
        c = math.factorial(n)/math.factorial(n-r)/math.factorial(r)
        return c



    def dmm(l,m1,m2,beta):
        '''

        compute dm1,m2(beta)  for  m1+m2>=0  and m1-m2>=0

        '''
        co = math.cos(beta/2.)
        si = math.sin(beta/2.)

        sum=0.
        for s in range(0,l-m1+1):
            var=0.
            var=combi(l+m2,l-m1-s)*combi(l-m2,s)*(-1.)**(l-m1-s)
            var=var*co**(2*s+m1+m2)*si**(2*l-2*s-m1-m2)
            sum=sum+var

        sum=sum*math.sqrt(math.factorial(l+m1)*math.factorial(l-m1)*1.)
        sum=sum/math.sqrt(math.factorial(l+m2)*math.factorial(l-m2)*1.)

        return sum


    def function_rot(l,beta):

        dim = 2*l+1
        mat = np.zeros((dim,dim))
        for i in range(0,l+1):
            for j in range(-i,i+1):
                  mat[i+l,j+l] = dmm(l,i,j,beta)

        for i in range(-l,1):
              for j in range(i,-i+1):
                  mat[i+l,j+l] = mat[-i+l,-j+l]*(-1.)**(i-j)

        for j in range(0,l+1):
            for i in range(-j,j+1):
                  mat [i+l,j+l] = dmm(l,j,i,-beta)

        for j in range(-l,1):
            for i in range(j,-j+1):
                  mat [i+l,j+l] = mat[-i+l,-j+l]*(-1.)**(i-j)

        return mat

    angle = math.pi*beta/180.
    zz = function_rot(l,angle)
    zz = zz[:,l]**2
    norm = zz.sum()
    return zz/norm

def activity_spectrum(f,parameters,verbose=0):
    """
    f : frequency values [muHz]
    parameters:  (sigma,tau) where  sigma: amplitude (in ppm) and tau: characteristic time (in days)
    An "single-sided power spectrum" is assumed

    """
    sigma= parameters[0] # in ppm
    tau =parameters[1]*86400. # characteristic time, converted from days to seconds
    H = 4.*tau*sigma**2*1e-6
    if(verbose):
        print (("activity parameters: sigma=%f [ppm], tau= %f [days] %f [s]") % (sigma,tau/86400.,tau))
    return plf(f,H,tau*1e-6,2.)

def granulation_spectrum(f,numax,verbose=0,type=0,mass=1.,teff=-1.,radius=-1.):
    
    '''
   Granulation spectrum
   
   GS,parameters = granulation_spectrum(numax,tau)
   We assume an "single-sided power spectrum"

   INPUTS:
   f : frequency values [muHz]
   numax: peak frequency [muHz]
   type :
   =0 -> assume standard scaling relations and a Lorentzian form
   =1 -> as in Kallinger et al (2014)
   mass : star mass (used for type=1)
   
   OUTPUTS:
   GS: the granulation spectrum computed for the frequency set f
   parameters: granulation parameters, the returned parameters depend on "type"

   type = 0 : parameters = (H,tau,sigma)
      H: Height of the granulation component [ppm2/muHz]
      tau: time-scale [s]
      sigma: amplitude of the granulation [ppm]

   type = 1 : parameters = (h1,tau1,c1,h2,tau2,c2)
   
    '''
    if (type==0):
        
        tau_gran =  9e4*(numax/10.)**(-0.90) # time scale [seconds]  ref. M13
        taue = tau_gran # e-folding type, taue = tau_gran for a Lorentzian function
        #    h_gran =  2e6*(tau_gran/1e5) **2.34 # height [ppm2/muHz]  ref.  M13
        h_gran = 4e5*(numax/10.)**(-2.15) # height [ppm2/muHz]  ref.  M13
        sigma_gran =  math.sqrt(h_gran*1e6/tau_gran/4.)
        if(verbose):
            print (("tau [s], H [ppm2/muHz], sigma [ppm]: %e, %e, %f, %g")%   (h_gran , tau_gran , sigma_gran,taue))
        gran_par = {"h1": h_gran, "tau1": tau_gran, "alpha1": 2.,"taue": taue}
        return plf(f,h_gran,tau_gran*1e-6,2.), gran_par
    else:
##        if (True):
        if ( (numax < 2.) | (numax > 3500.)):
            print ("Outside the valid numax domain of Kallinger et al (2014) scaling relations for the granulation spectrum")
            print ("Assuming Samadi et al (2013)'s scaling relations")
            if( (radius <0) | (mass <0.) | (teff<1)):
                raise SLSError("mass, teff and radius must be specified")
            Ma0 = 0.258
            taue_sun = 1.72e2 * 1e-6 # / 10^6 s
            logg = math.log10(mass/radius**2) + logg_sun
            Ma = Ma0 * (teff/teffsun)**2.35 * (10.**(logg -logg_sun))**(-0.152)
            taue = taue_sun*(numaxref/numax)*(Ma0/Ma) 
            fg = -0.67 + 2.3*(Ma/Ma0)  -0.59*(Ma/Ma0)**2
            sigsun = 38.6	
            sig2 = sigsun**2 * (teff/teffsun)**(3./2.)*(1./mass)*(numaxref/numax)*fg**4  # Eq. 14
            Cbol = (teff/teffsun)**0.8
            h = sig2*taue*4./Cbol**2
            if(verbose):
                print (("Ma, fg, sig2, taue; Cbol, H : %f, %f, %f, %f, %f, %f") % (Ma,fg,sig2,taue,Cbol,h))
            
            gran_par = {"h": h, "tau": taue, "alpha": 2,"taue": taue}
            
            return  plf(f,h,taue,2) , gran_par

        a1k = 3382
        a1s = -0.609
        a1t = 0.

        a2k = 3710.
        a2s = -0.613
        a2t = -0.26

        b1k = 0.317
        b1s = 0.970

        b2k = 0.948
        b2s = 0.992

        c1 = 4.
        c2 = 4.
	
        a1 = a1k*numax**a1s*mass**a1t
        a2 = a2k*numax**a2s*mass**a2t

        b1 = b1k*numax**b1s
        b2 = b2k*numax**b2s


        xi1 = 2.*math.sqrt(2.)/math.pi
        tau1   = 1./(2.*math.pi)/b1

        xi2 = xi1
        tau2   = 1./(2.*math.pi)/b2

        h1 = a1**2/b1*xi1
        h2 = a2**2/b2*xi2
        taue = eft_2plf(h1,tau1,c1,h2,tau2,c2) # e-folding time
        gran_par = {"h1": h1, "tau1": tau1, "alpha1": c1,"h2": h2, "tau2": tau2, "alpha2": c2,"taue": taue}
        if(verbose):
            print (("h1 [ppm2/muHz], tau1 [s], c1, h2 , tau2 , c2: %g, %g, %g, %g, %g, %g, %g") % (h1,tau1*1e6,c1,h2,tau2*1e6,c2,taue))

        return  plf(f,h1,tau1,c1) + plf(f,h2,tau2,c2),gran_par

def amax(numax,deltanu,teff):
    '''

    Scaling relation for the oscillation maximum (Amax) as established by Corsaro et al (2013, MNRAS 430, 2313-2326)

    We assume the model named  4\beta (see Table 3 & 4)
    
    If numax >150 muHz,  coefficients established using the short cadence data (table 3) are assumed otherwise those obtained with the long cadence data (table 4).
    
    
    Return the value of Amax bolometric (ppm) as a function of numax (muHz), deltanu (muHz) and teff (K)
    

    '''

    Amaxbolsun = 2.53 # rms value (see Michel et al 2009)


    if ( (numax > 4000.) | (numax < 2.) ):
        print ("WARNING ! numax outside the domain for the oscillation maximum scaling relation of  Corsaro et al (2013), assuming Samadi et al (2007)'scaling relation")
        Amaxbol = Amaxbolsun * (teff/teffsun)**(7./2.)*(numaxsun/numax)
        return Amaxbol

    if( numax > 150.):
        # parameters of model 4\beta for short cadence data
        s = 0.748
        r = 3.47
        t = 1.27
        lnbeta = 0.321

        ## model M1 
        ## s = 0.624
        ## r = 3.68
        ## lnAmax = -s*math.log(numax/numaxsun) + (3.5*s -r + 1.) *math.log(deltanu/deltanusun)
        ## Amaxbol = math.exp(lnAmax) * (teff/5934.)**(0.8) *Amaxbolsun
    else:
        # parameters of model 4\beta for long cadence data
        s = 0.602
        r = 5.87
        t = 1.32
        lnbeta = 0.45

    lnAmaxbol = (2.*s-3.*t)*math.log(numax/numaxsun) + (4.*t-4.*s)*math.log(deltanu/deltanusun) + (5.*s-1.5*t-r+0.2)*math.log(teff/teffsun) + lnbeta

    Amaxbol = math.exp(lnAmaxbol)*Amaxbolsun

    return Amaxbol


def lwmax(teff,numax):
    '''
    Scaling relation for the mode linewidth at numax as established by Appourchaux et al (2012, A&A, 537, A134)

    Return the mode linewidth (muHz) at numax as a function of teff (K)
    
    '''
    if ( (teff <= 6700.) & (teff >= 5300.) ):
        # Appourchaux et al (2012), table 2, at maximum mode height
        Gamma0 = 0.20
        alpha = 0.97
        s = 13.0
        return  Gamma0 + alpha*(teff/teffsun)**s
    else:
        if( (teff<4000.) | (teff>6700.)):
            print("Warning !! Teff outside the domain for the mode line-width scaling relations")
##    elif( (teff<=5300.) & (teff>=4000.)):
        # Belkacem, 2012, A&A
        numax0 = 909.253761301
        return  0.19*(teff/4800.)**(10.8-0.3*0.5)*(numax/numax0)**(-0.3) #
##        return  0.19*(teff/4800.)**(10.8)
    
        
    

def get_obs_par(mag, T, dt , pn_ref = 7.7, mag_ref = 6.,wn_ref = 0. , verbose = False, teff = 4800.):
    

    # total white-noise level ppm.s^(1/2)
    W = math.sqrt ( pn_ref**2 * 3600.*10.**((mag-mag_ref)/2.5)  + wn_ref**2    * 3600.*10.**(2.*(mag-mag_ref)/2.5) )
    if(verbose):
        print ('Total white-noise [ppm.Hz^(-1/2)]: %e' % W)
        print ('Total white-noise at sampling time: %e' % (W*math.sqrt(1./dt)))
 ## 2.*W**2*1e-6

    T0 = T*86400. # duration in seconds
    
    # size of the time series
    n = int(T0/(dt))

    # frequency resolution [muHz]
    dnu = 1./(n*dt)*1e6


    nyq = 1./(2.*dt)*1e6 # Nyquist frequency [muHz]

    m2b = (teff/5934.)**(0.8) # conversion from  measured to bolometric amplitude (Ballot et al 2011)

    # observation parameters
    obs_par = {'magnitude': mag, 'duration': T, 'sampling': dt, 'frequency_resolution': dnu, 'nyquist': nyq, 'white_noise': W, 'time_steps_number': n , 'bolometric_coefficient': m2b}
    
    return obs_par


def read_agsm(file):
    '''
     modes,starpar = read_agsm(file)

    
     OUTPUTS:
     modes : mode parameters:
     
     modes[:,0] : n order
     modes[:,1] : l degree
     modes[:,2] : nu frequency in muHz (eigenvalue)
     modes[:,3] : square normalised frequency (w2)
     modes[:,4] : Richardson frequency in muHz
     modes[:,5] : variationnal frequency in muHz
     modes[:,6] : icase
     modes[:,7] : mode inertia g cm^2  x 10^40
     modes[:,8] : Ledoux coefficient

     starpar : star parameters (mass,radius)
     
     
    '''

    f = FortranIO.FortranBinaryFile(file, mode='rb')

    cs = []
    while True:
        try:
            csi = f.readRecordNative('d')
            cs.append(csi)
        except IndexError:
            break

    nmodes = len(cs)
    modes = np.zeros((nmodes,9))
    mstar = (cs[0])[1]
    rstar = (cs[0])[2]
    for i in range(nmodes):
        modes[i,0] = (cs[i]) [18] # n
        modes[i,1] = (cs[i]) [17] # l        
        sigma2 = (cs[i]) [19]
        omega2 = sigma2*ggrav*mstar/(rstar**3)
        modes[i,2] = math.sqrt(omega2)/(2.*math.pi)*1e6 # nu in muHz
        modes[i,3] = sigma2
        modes[i,4] = (cs[i]) [36]*1e3 # Richardson frequency in muHz
        modes[i,5] = (cs[i]) [26]*1e3 # variational frequency in muHz
        modes[i,7] =  (cs[i]) [23] # * (mstar*1e-20) * ( (cs[i])[29 ] *  rstar*1e-10  )**2 *  4. *math.pi 
        modes[i,8]  = (cs[i]) [35] 
    
    f.close()
    # reading separately the 'ics' integer variables
    f = FortranIO.FortranBinaryFile(file, mode='rb')
    n = 38*2
    for i in range(nmodes):
        data = f.readRecordNative('i')
        ics = data[n:n+8]
        modes[i,6] = ics[4]

    f.close()

    return modes,(mstar/msun,rstar/rsun)

def osc_spectrum(starID, f,modes,numax,deltanu,teff,verbose=False,a=0.,b=1.,plot=False,win=0, rot_period_sur = -1., incl = 0., path='./'):
    # a,b  surface effect parameters (Lorentzian function)
    # rot_period_sur : period of the surface rotation, in days. Do not include splittings if rot_period_sur  < 0

    starName = ("%10.10i") % starID
    Gamma =lwmax(teff,numax)
    Amax = amax(numax,deltanu,teff)
    Hmax = 2.*Amax**2/(math.pi*Gamma) # Mode height at numax assuming resolved modes (from Lochard et al 2005, A&A, 438, 939, eq. 16)
    # the additional factor comes from the fact that Lochard et al 2005 assumed two-side power density spectrum
    # here we assume an single-sided power density spectrum
    nm = modes.shape[0]
    width = 0.66*(numax**0.88)        # dnu_env = FWHM of the excess power envelope, see: Mosser et al. (2012 A&A 537 A30) - Table 2

    if(verbose):
        print ('------------------------------------------------------------------------')
        print(('Amax=',Amax,' [ppm]'))
        print(('Gamma=',Gamma, ' [muHz]'))
        print(('Hmax=',Hmax,' [ppm^2/muHz]'))
        print(('Width=',Hmax,' [muHz]'))
        print ('theoretical input frequencies:')
        print( "n, l, nu_eig, nu_var , nu_eig-nu_var,nu_eig-nu_richardson inertia")
        u = modes[:,6] == 10010
        for i in range(nm):
            if (u[i]):
                print (("%i %i %f %f %f %f %e") % (modes[i,0],modes[i,1],modes[i,2],modes[i,5],modes[i,2]-modes[i,5],modes[i,2]-modes[i,4],modes[i,7]))
        print ('------------------------------------------------------------------------')
    spec = np.zeros(len(f))
    df = f[1] - f[0]
    # square visibility


    V1 = 1.5
###    V1 = 0.2
####   ------------------------------------

    V2 = 0.5
    V3 = 0.05
    # 
    V = (1.,V1,V2,V3)

### !!! A ADAPTER POUR LES MS !!!

    # identification of the radial modes 
    ir = ( modes[:,1] ==0)
    nr = ir.sum()
    Imax = np.interp(numax,modes[ir,2],modes[ir,7])
    NU = []
    L = [] # angular degrees
    LM = []  # azimuth degree
    N = []
    G = []
    H = []
    I = []
    DNU = [] 
    gauss = lambda nu : np.exp(-((nu-numax)**2)/((2.*width)**2/(4*math.log(2))))
    ## A = 2.
    Ap = 2.
    An = 6.
    fout = open(path+starName +  '.modes','w')
    fout.write("# n, l, m , nu, gamma, h, I/Imax, dnu, splitting\n")

    for i in range(nm): # loop over the modes (n,l)
        inertia = modes[i,7]
        nu0 = modes[i,2] #  eigen frequency i
        n = int(modes[i,0])
        ell = int(modes[i,1])
        if (rot_period_sur > 0):
                nlm = 2*ell + 1 # total numver of the multiplets 
                lm1 = -ell
                lm2 = ell
        else:
                nlm = 1
                lm1 = 0
                lm2 = 0
        if (rot_period_sur <= 0.): 
            pr = np.ones(nlm)
        else:
            pr = power_ratio(ell,incl)   
        for lm in range(lm1,lm2+1): # loop over the multiplets 
            if (rot_period_sur <= 1e-10):
                split = 0.    
            else:
                split = lm/(rot_period_sur*86400.)*1e6*modes[i,8]
##        print lm,modes[i,8]
            icase = int(modes[i,6])
    ##        dnu = a*numax*(nu/numax)**b # Kjeldsen & Bedding 2008
    ##        dnu = numax * (c + a/(1. + (nu/numax)**b) )
            nu = nu0 + split
            dnu = a* numax * (1. -  1./(1. + (nu/numax)**b) ) # surface effects, Eq. 9, Sonoi et al 2015, A&A, 583, 112
            nu = nu + dnu  # we add surface effects
            h = gaussenvelop(nu,Hmax,numax,width)*V[ell]*pr[lm-lm1]
##            gamma = Gamma*(Imax/inertia)* (A - (A-1.)*gauss(nu))
            gamma = Gamma*(Imax/inertia)* (1. + (Ap*(nu>=numax) + An*(nu<numax) )* (1. - gauss(nu)))
            if (icase != 10010):
##                if(verbose):
##                    print 'excluded mode according to icase=',icase
                continue
            '''
            if (n  <= 0):
##                if(verbose):
##                    print 'excluded mode because n<=0 : ',n
                continue
            '''
            if (ell  > 3):
##                if(verbose):
##                    print 'excluded mode because l>3 : ',ell
                continue
            fout.write( ("%i %i %i %f %e %e %e %f %f\n") % (n,ell,lm,nu,gamma,h,inertia/Imax,dnu,split))
            if ( gamma > df*2.): # resolved profile
                P =  1./ ( 1. + ( 2.*(f-nu)/gamma)**2 )   # Lorentzian : modes shape
            else: # un-resolved profile
                P = (math.pi*gamma/2./df)*( np.sinc( math.pi*(f-nu)/df) )**2
            spec[:] = spec[:] + P * h
            NU.append(nu)
            L.append(ell)
            LM.append(lm)
            N.append(n)
            G.append(gamma)
            H.append(h)
            I.append(inertia/Imax)
            DNU.append(dnu)
        
    N = np.array(N)
    L = np.array(L)
    NU = np.array(NU)
    G = np.array(G)
    H = np.array(H)
    I = np.array(I)
    DNU =  np.array(DNU)
    if(plot):
        import matplotlib
##        matplotlib.use('Agg')
        import matplotlib.pyplot as plt

        plt.close('all')
        plt.figure(win)
        plt.clf()
        plt.plot(NU,G*I,'ro')
        plt.xlabel(r'$\nu$ [$\mu$Hz]')
        plt.ylabel(r'$\Gamma (I/I_{\rm max})$ [$\mu$Hz]')
        plt.semilogy()
        plt.title(starName)
        plt.savefig(starName +  ("_%2.2i.pdf") % (win) )

        win += 1
        plt.figure(win)
        plt.clf()
        plt.plot(NU,G,'ro')
        plt.xlabel(r'$\nu$ [$\mu$Hz]')
        plt.ylabel(r'$\Gamma$ [$\mu$Hz]')
        plt.semilogy()
        plt.title(starName)
        plt.savefig(starName +  ("_%2.2i.pdf") % (win) )

        win += 1
        plt.figure(win)
        plt.clf()
        plt.plot(NU,I,'ro')
        plt.xlabel(r'$\nu$ [$\mu$Hz]')
        plt.ylabel(r'$I/I_{\rm max}$')
        plt.semilogy()
        plt.title(starName)
        plt.savefig(starName +  ("_%2.2i.pdf") % (win) )

        win += 1
        plt.figure(win)
        plt.clf()
        plt.plot(NU,H,'ro')
        plt.xlabel(r'$\nu$ [$\mu$Hz]')
        plt.ylabel(r'$H$ [ppm$^2/\mu$Hz] ')
        plt.semilogy()
        plt.title(starName)
        plt.savefig(starName +  ("_%2.2i.pdf") % (win) )

        win += 1
        plt.figure(win)
        plt.clf()
        plt.plot(NU,DNU,'ro')
        plt.xlabel(r'$\nu$ [$\mu$Hz]')
        plt.ylabel(r'$\delta \nu$ [$\mu$Hz]')
        plt.title(starName)
        plt.savefig(starName +  ("_%2.2i.pdf") % (win) )
    
        plt.show(block=False)
        ## s=raw_input('Type ENTER to continue')
    return spec,(Amax,Hmax,Gamma,width),(N,L,LM,NU,G,H,I,DNU)

def gen_adipls(starID, filename, teff, dt,T, mag , seed = None, pn_ref = 7.7, wn_ref = 0. , mag_ref = 6.,verbose = False, GST = 1 , a=0 , b=1.  ,  plot=False, rot_period_sur = -1., incl = 0., activity = None, granulation = True , path= './' , oscillation=True):
    '''

    (time,lc,nu,pds,mpds,par,seed) = gen_adipls( starID, filename, teff, dt,T, mag , seed = None, pn_ref = 7.7, wn_ref = 0. , mag_ref = 6., verbose = False, GST = 1 , a=0 , b=1.  ,  plot=False) 
    
    Generate a simulated light-curve using eigenfrencies computed with ADIPLS

    INPUTS:

    starID : any number to identify the simulation, this is used only for the plots and the generated output files 
    filename: name of the grand summary file (.gsm) generated by ADIPLS
    teff: effective temperature [K]
    dt: sampling time [s]
    T: simulation duration [days]
    mag: magnitude of the star
    seed: seed of the Random Number Generator
    pn_ref: reference value of the photon noise [in ppm per hour]
    wn_ref: reference value of the non-photonic white noise  [in ppm per hour]
    mag_ref: reference magnitude for which pn_ref and wn_ref are specified

    GST: type of granulation spectrum (see the function named 'granulation_spectrum')
    a,b,c: coefficients involved in the formulation used of the surface correction
    plot: if True  do some plot
    verbose: if True be verbose
    rot_period_sur : period of the surface rotation, in days. Do not include splittings if rot_period_sur  < 0
    incl : inclination angle (in degrees)

    activity: add an activity component, tuple (sigma,tau) where  sigma is the amplitude (in ppm) and  tau the characteristic time (in days) ; activity= None  if no activity is added (default case)
    granulation: add the granulation component (by default: True)
    oscillation: turn on oscillation (default: True)    


    path: where to save the information about the modes

    OUTPUTS:
    time : time vector (in seconds)
    lc: ligth-curve
    nu: frequencies [muHz]
    pds: power density spectrum [ppm^2/muHz]
    mpds: mean power density spectrum  [ppm^2/muHz]
    par: adopted paramaters of the simulation
    seed: adopted value of the seed
    
    '''
    
    obs_par =  get_obs_par(mag, T, dt , pn_ref = pn_ref,  wn_ref = wn_ref, mag_ref = mag_ref,verbose = verbose, teff=teff)

    dnu = obs_par['frequency_resolution']
    n = obs_par['time_steps_number']
    W = obs_par['white_noise']
    nyq =  obs_par['nyquist']
    
    # frequency values [muHz], n-1 values, null frequency is excluded
    f = (np.arange(n/2-1)+1)*dnu
    b2m = 1./ obs_par['bolometric_coefficient'] # bolometric to measured amplitude coefficient

    modes, (mass, radius) = read_agsm(filename)

    star_par = {'mass': mass, 'teff' : teff, 'radius' : radius, 'logg': math.log10(mass/radius**2) +logg_sun} 
  
    numax =  numaxref * (mass/radius**2) * math.sqrt(teffsun/teff)
##    deltanu = deltanusun * (mass/radius**3)
    deltanu = deltanusun * math.sqrt(mass/radius**3)
    if(verbose):
    	print (('numax = %f, deltanu = %f [muHz]') % (numax,deltanu))

    OS,(Amax,Hmax,Gamma,width_env),modespar = osc_spectrum(starID, f,modes,numax,deltanu,teff,verbose=verbose,a=a,b=b, plot=plot, rot_period_sur = rot_period_sur , incl = incl , path = path )

    # Granulation spectrum (GS)
    if(granulation):
        GS, gran_par   = granulation_spectrum(f,numax,mass=mass,teff=teff,radius=radius,type=GST) 
        GSf,  gran_par  = granulation_spectrum(2.*nyq-f,numax,mass=mass,teff=teff,radius=radius,type=GST,verbose=verbose)  # Folded component of the GS
    else:
        GS,GSf = 0.,0.
        gran_par = {}

    if activity is None:
        AS = 0.
        ASf = 0.
        activity_par = {'sigma': 0., 'tau': 0.}
    else:
        AS = activity_spectrum(f,activity,verbose=verbose)
        ASf = activity_spectrum(2.*nyq-f,activity)
        activity_par = {'sigma': activity[0], 'tau': activity[1]}
        
    # mean power spectrum [ppm2/Hz]
    mpsd =  ( (OS * (b2m**2) * ( (oscillation == True)*1.) + GS  + GSf + AS + ASf ) * 1e6  *   (np.sinc(0.5*f/nyq))**2   +  2.*W**2 )  # [ppm^2/Hz] 

    time, ts , ps , seed =  mpsd2rts(f*1e-6,mpsd,seed=seed)

##    osc_par = (numax,deltanu,Amax,Hmax,Gamma,width_env,a,b,c)
    osc_par = {'numax' : numax, 'deltanu': deltanu, 'Amax' : Amax, 'Hmax' : Hmax, 'Gammamax': Gamma, 'width_envelop' : width_env, 'se_a': a, 'se_b': b }
   
    if(verbose):
        print (("numax [muHz], delta_nu [muHz], mass, radius, Teff [K]: %f , %f, %f, %f, %f") % (numax,deltanu,mass,radius,teff))
        print (("Hmax [ppm2/muHz], Amax [ppm], gamma [muHz], width_env [muHz]: %e , %f , %f, %f") % (Hmax,Amax,Gamma,width_env))
        print (("a= %f, b= %f") % (a,b))
        print (("b2m, white_noise  [ppm2/muHz]: %f %g")% ( b2m,2.*W**2*1e-6 ))
        print (("times series, standard deviation [ppm]: %f") % np.std(ts))
        print (("theoretically expected standard deviation [ppm]: %f") %  math.sqrt(  np.sum(mpsd)*1e-6* f[0] )) 

    # output parameters
    opar = (star_par,obs_par,osc_par,gran_par,activity_par)

    return time,ts,f,ps*1e-6,mpsd*1e-6,opar,seed


def gen_up(starID, numax,dt,T, mag , delta_nu =  -1. , mass = -1. , seed = None, pn_ref = 7.7, wn_ref= 0., mag_ref = 6.,verbose = False, teff = 4800., DPI = -1.,  q = 0. , GST = 0 , incl = -1 , rot_f = 0., path= '', granulation = True , oscillation=True):
    '''
    Generate a simulated light-curve on the basis of the Universal Pattern
    
    time,ts,f,ps,mps,opar,seed = gen_up(numax,dt,T, mag, delta_nu =  -1. ,teff = 4800., DPI = -1.,  q = 0.,pn_ref = 7.7,mag_ref = 6., seed = None,verbose= False)
    
    INPUTS:

    starID : any number to identify the simulation, this is used only for the plots and the generated output files 
    numax : peak frequency [muHz]
    dt : sampling time [s]
    T:  duration of the time series [days]
    mag: star magnitude

    delta_nu :  mean large separtion [muHz], if negative, delta_nu is computed according to a scaling relation
    teff : effective temperature [K], default: 4800 K
    DPI : asymptotic period spacing [s], negative if no mixed modes are wanted
    q : coupling coefficient, default: q=0 (no coupling)

    pn_ref: photon-noise reference level  [ppm per hour]
    wn_ref: Non photonic white-noise reference level  [ppm per hour]
    mag_ref: reference magnitude for the white-noise levels
    seed: seed of the Random Number Generator (RNG)
    verbose : verbose mode if True


    OUTPUTS:
    time: the time values
    ts: the simulated time-series (flux fluctuations in ppm)
    f: frequency values [muHz]
    ps: simulated power spectrum [ppm2/muHz]
    mps: mean power spectrum [ppm2/muHz]
    opar: simulation parameters
    seed:  adopted value of the seed (chosen by the Random Number Generator if seed is not specified in input)

    Simulation (output) parameters:
    - H_env : height of the oscillation envelope  [ppm^2/muHz]
    - width_env : width of the envelope [muHz]
    - gamma : mode width (for pure acoutic modes) [muHz]
    - h_gran : height of the granulation  component
    - tau_gran : time-scale of the granulation
    - sigma_gran : rms brightness fluctuations associated with the granulation [ppm]
    - b2m : bolometric to measured flux conversion factor
    parameters = (numax,delta_nu,mass,teff,H_env,width_env,gran_par,b2m,wn)    

    '''

    obs_par =  get_obs_par(mag, T, dt , pn_ref = pn_ref,  wn_ref = wn_ref, mag_ref = mag_ref,verbose = verbose, teff=teff)

    dnu = obs_par['frequency_resolution']
    n = obs_par['time_steps_number']
    W = obs_par['white_noise']
    nyq =  obs_par['nyquist']

    # frequency values [muHz], n-1 values, null frequency is excluded
    f = (np.arange(n-1)+1)*dnu
    f = (np.arange(n/2-1)+1)*dnu
    b2m = 1./ obs_par['bolometric_coefficient'] # bolometric to measured amplitude coefficient
    
    # Mean large separation
    if (delta_nu <=0. ):
        delta_nu = 0.274*numax**0.757 # ref. M13

    if (mass <=0.):
        # mass estimated combining the "classical" scaling laws for numax et deltanu
        mass = (numax/numaxref)**3. * (delta_nu/deltanusun)**(-4.) * (teff/teffsun)**(3./2.)

    radius = (numax/numaxref)*(delta_nu/deltanusun)**(-2) *(teff/teffsun)**(1./2.)

    # H_env = 2.3* 2.03e7*numax**(-2.38) # [ppm2/muHz], Mosser sf2a 2013 , mutlipied by 3 to have the true Hmax (??)
    H_env = 2.01e7 *numax**(-1.9) # [ppm2/muHz],  Peralta et al 2018
        
    
    # Mode line width, from Vrard et al 2018
    if( (mass >= 1.0) & (mass<1.2)):
        gamma = 0.1*(teff/4800.)**(4.55)
    elif( (mass >= 1.2) & (mass<1.4)) :
        gamma = 0.1*(teff/4800.)**(5.29)
    elif( (mass >= 1.4) & (mass<1.6)) :
        gamma = 0.1*(teff/4800.)**(4.54)
    elif( (mass >= 1.6) & (mass<1.8)) :
        gamma = 0.1*(teff/4800.)**(4.65)
    else:
        print ("UP:  scaling for mode line-width (Vrard et al 2018): stellar mass M=%g is out of range M=[1.2-1.8]" % (mass))
        print ("Belkacem et al's theoretical scaling relation is adopted")
        gamma = 0.19*(teff/4800.)**(10.8)   # Belkacem, sf2a, 2012

    
    width_env =  0.66*(numax**0.88)        # dnu_env = FWHM of the excess power envelope, see: Mosser et al. (2012 A&A 537 A30) - Table 2
    
    # building The Universal Pattern
    StarName =  ("%10.10i") % starID
    fout = open(path + StarName + '.modes','w')
    UPm,_ = UP.universal_pattern(delta_nu, numax, H_env,  f , nyq , verbose = verbose , teff = teff, DPI = DPI, q=q, gamma=gamma, width = width_env , rot_f = rot_f , beta = incl , epsg = 0.25 , fout = fout)
    fout.close()

    # Granulation spectrum (GS)
        # Granulation spectrum (GS)
    if(granulation):
        if ( (numax < 2.) | (numax > 3500.)):
            raise SLSError("Numax must be in the range of 2-3500 uHz if model is UP! ")
        GS, gran_par   = granulation_spectrum(f,numax,mass=mass,type=GST) 
        GSf,  gran_par  = granulation_spectrum(2.*nyq-f,numax,mass=mass,type=GST,verbose=verbose)  # Folded component of the GS
    else:
        GS,GSf = 0.,0.
        gran_par = {}

    
            

    # mean power spectrum [ppm2/Hz]
    mpsd =   (UPm * (b2m**2) * ( (oscillation == True)*1.)  + GS  + GSf  ) * 1e6 *(np.sinc(0.5*f/nyq))**2   +  2.*W**2   #  

    time, ts , ps , seed =  mpsd2rts(f*1e-6,mpsd,seed=seed)

    star_par = {'mass': mass, 'teff' :  teff, 'radius': radius}
    osc_par = {'numax' : numax, 'deltanu': delta_nu, 'Hmax' : H_env, 'Gammamax': gamma, 'width_envelop' : width_env,'inclination': incl, 'rot_f': rot_f }
    activity_par = {'sigma': 0., 'tau': 0.}
    
    if(verbose):
        print (("numax [muHz], delta_nu [muHz], mass, Teff [K]: %f , %f, %f, %f") % (numax,delta_nu,mass,teff))
        print (("H_env [ppm2/muHz], width_env [muHz], gamma [muHz]: %e , %f , %f") % (H_env,width_env,gamma))
        print (("b2m, white_noise  [ppm2/muHz]: %f %g")% ( b2m,2.*W**2*1e-6 ))
        print (("times series, standard deviation [ppm]: %f") % np.std(ts))
        print (("theoretically expected standard deviation [ppm]: %f") %  math.sqrt(  np.sum(mpsd)*1e-6* f[0] )) 

    # output parameters
    opar = (star_par,obs_par,osc_par,gran_par,activity_par)

    return time,ts,f,ps*1e-6,mpsd*1e-6,opar,seed



def savelc(time,lc,StarID,par,seed,path='',navg=1,lctype=0):
    '''

    savelc(time,lc,StarID,par,seed,path='')
    
    Save the light-curve in an ASCII file compatible with kepler and the SSI pipeline (TypeFile=8)

    time : time values
    lc : the light-curve
    StarID : ID of star (StarID must be >0)
    par : simulation parameters 
    seed: seed associated with the simulation
    path: where to save the ligth-curve
    navg: input light-curve values are averaged over navg consecutive values (default: navg = 1, i.e. no average)
          the sampling of the output light-curves will then be equal to dt*navg
    lctype: type of ligt-curve format, 
        type=0 : KASOC ASCII format
        type=1 : simple ASCII format
    '''


    if (navg > 1 ):
        n = lc.size/navg
        t = rebin1d(time[0:n*navg],n)/float(navg)
        lc = rebin1d(lc[0:n*navg],n)/float(navg)
    else:
        t = time
        n = lc.size
    if(lctype==0):
        t = t/86400. # time in Julian days
    filename = path
    if(lctype==0):
        filename += ('kplr%10.10i') %  (StarID)
    else:
        filename += ('%10.10i') %  (StarID)

    f=open(filename+'.dat','w')
    f.write(('# seed  %i\n') % (seed))
    def ppar(par):
        n = len(par)
        i = 0 
        for u in list(par.items()):
            f.write(' %s = %g' % u)
            if (i < n-1):
                f.write(', ')
            i += 1
    f.write('\n')
    f.write('# star parameters:')
    ppar(par[0])
    f.write('# observations parameters:')
    ppar(par[1])
    f.write('# oscillation parameters:')
    ppar(par[2])
    f.write('# granulation parameters:')
    ppar(par[3])
    f.write('# activity parameters:')
    ppar(par[4])
    f.write('# navg: %i\n' % (navg))
    if(lctype==0):
        f.write('# Time [julian days], Flux [ppm], Flag\n')
        for i in range(n):
            f.write( ("%25.15e %25.15e   0\n") % (t[i],lc[i]) )
    else:
        f.write('# Time [s], Flux\n')
        for i in range(n):
            f.write( ("%25.15e %25.15e\n") % (t[i],lc[i]*1e-6+1.) )
        
        
    f.close()
  

def genslc(n,f,mpsd,par, seed=None,path='', verbose=False,navg = 1):
    '''    
    Generate several light-curves and save them in a set of files
    Each simulated light-curve corresponds to a given realization.
    
    seed  = genslc(n,numax,seed=None,path='', .... )

    n: number of simulated light-curves 
    f: frequency values [muHz] (as returned by gen_up or gen_adipls)
    mps: mean power spectrum [ppm2/muHz]  (as returned by gen_up or gen_adipls)
    par: simulation parameters (as returned by gen_up or gen_adipls)

    path: directory where the light-curves will be stored
    seed: initial seed of the Random Number Generator (RNG). If seed=None, choose one randolmy
    verbose: if true, verbose mode is on
        navg: light-curve values are  averaged over navg consecutive values (default: navg = 1, i.e. no average). 
              the sampling of the output light-curves will then be equal to dt*navg


    Example:
    seed = 13434343
    time,ts,f,ps,mps,par,seed = gen_up(StarID, numax,dt,T, mag , seed = seed )
    seed = genslc(n,f,mps,par,seed=seed)

    '''

    if( seed == None):
        # see is not specified, we choose a random value 
        seed = np.random.random_integers(0,high=2**30)    
    # initialization of the state of the RNG
    np.random.seed(seed)
    seeds = np.random.random_integers(0,high=2**30,size=n)
    for i in range (n):
        StarID = i+1
        time, ts , ps, seedi =  mpsd2rts(f*1e-6,mpsd*1e6,seed=seeds[i])

        savelc(time,ts,StarID,par,seedi,path=path,navg=navg)

    return seed


def ExtractSystematicDataMagRangeV2(filename,Vp,DriftLevel='medium',Verbose=False,seed=None):
    '''
    Vp: PLATO V reference magnitude (i.e.  V@T=6000K)
    '''
    if(Verbose):
            print('Loading the file %s that contains the parameters associated with the residual systematic error ' % filename)
    Data = np.load(filename,encoding="latin1",allow_pickle=True)
    Mag = Vp - 0.34 # conversion into P magnitude
    magcorr = 0.
    if(not (DriftLevel is None)):
        if(DriftLevel == 'low'):
            DriftAmpRange = [0.,0.4]
        elif(DriftLevel == 'medium'):
            DriftAmpRange = [0.4,0.8]
        elif(DriftLevel == 'high'):
            DriftAmpRange = [0.8,1e3]
        elif(DriftLevel == 'min'):
            DriftAmpRange = None
        elif(DriftLevel == 'max'):
            DriftAmpRange = None
        else: # any amplitude
            DriftAmpRange = [0.,1e3]
    n = 0
    DataOut = []
    try:
        if(Verbose):
            print('%i cameras simulated' % len(Data))    
        DataC1 = Data[0]
        p = len(DataC1) 
        IDC1 = np.zeros(p,dtype=np.int)
        MagC1 = np.zeros(p)
        MaxDispC1 = np.zeros(p)
        i = 0
        for d in DataC1:
            IDC1[i] =  int(d['StarID'])  
            MagC1[i] =  d['Mag']  
            MaxDispC1[i] =  d['MaxDisp']  
            i+= 1
        if(Verbose):
            print('%i stars included' % i)
        if(DriftLevel == 'min' or DriftLevel == 'max'):
            m =  (MagC1+magcorr-0.25<=Mag) & (Mag<MagC1+magcorr+0.25)
            if(DriftLevel == 'min'): 
                v = np.min(MaxDispC1[m])
            else:
                v = np.max(MaxDispC1[m])
            m = m & (np.abs(MaxDispC1-v) <1e-5)
        else:
            m =  ((MagC1+magcorr-0.25<=Mag) & (Mag<MagC1+magcorr+0.25)  & (MaxDispC1>=DriftAmpRange[0]) & (MaxDispC1<DriftAmpRange[1]))
        nm = m.sum()
        if(Verbose):
            print('%i stars are matching the criteria' % nm)
        if(nm==0):
            raise SLSError('None of the stars in %s are matching the criteria' % filename)   
        np.random.seed(seed)
        j = np.random.randint(0, nm)
        StarIDData = IDC1[m][j]
        if(Verbose):
            print('Among them, StarID %9.9i has been randomly selected (Magnitude P= %f, Vp= %f) ' % (StarIDData,MagC1[m][j],MagC1[m][j]+0.34))
        ic = 0
        for C in Data:
            for s in C:
                if (s['StarID'] == StarIDData):
                    DataOut.append(s) 
                    if(Verbose):
                        SPRtot = s['SPRtot']
                        MaxDisp = s['MaxDisp']
                        MaskData = s['MaskData']
                        if(MaskData is not None):
                            MaskUpdNb = MaskData['MaskUpdNb']
                            print('Camera #%i: SPRtot = %f Max drift = %f, Mask number: %i' % (ic+1,SPRtot,MaxDisp,MaskUpdNb))
                        else:
                            print('Camera #%i: SPRtot = %f Max drift = %f' % (ic+1,SPRtot,MaxDisp))
                    n +=1
            ic += 1
        if(Verbose):
            print('%i individual light-curves for StarID %9.9i were found in %s ' %(n,StarIDData,filename))
    except SLSError:
        raise SLSError('Some errors occurs with %s' % filename)
    except:
    ## except KeyError:
        raise SLSError('Some errors occur with %s , are you sure about the data version (Systematics/Version parameter) ?' % filename)

    return DataOut


def ExtractSystematicDataMagRange(filename,Vp,version=2,DriftLevel='medium',Verbose=False,seed=None):
    '''
    Vp: PLATO V reference magnitude (i.e.  V@T=6000K)
    '''
    
    if(version ==0):
        magcorr = 0.218 # magnitude correction : original reference flux 2.64e5 e-/exp, corrected : 2.16e5, ratio : 1.222
        Mag = Vp
    elif(version ==1):
        magcorr = 0.
        Mag = Vp - 0.34 # conversion into P magnitude
        if(not (DriftLevel is None)):
            if(DriftLevel == 'low'):
                DriftAmpRange = [0.,0.4]
            elif(DriftLevel == 'medium'):
                DriftAmpRange = [0.4,0.8]
            elif(DriftLevel == 'high'):
                DriftAmpRange = [0.8,1e3]
            else: # any amplitude
                DriftAmpRange = [0.,1e3]
    elif(version==2):
        return ExtractSystematicDataMagRangeV2(filename,Vp,DriftLevel=DriftLevel,Verbose=Verbose,seed=seed)

    else:
        raise SLSError('Unknown version of file for the systematic error parameters')
    

    n = 0
    DataOut = []
    try:
        Data = np.load(filename,encoding="latin1",allow_pickle=True)
        if(version <1):
            for d in Data:
                if( (d['Mag']+magcorr-0.25<=Mag) and (Mag<d['Mag']+magcorr+0.25)):
                    DataOut.append(d)
                    n += 1
        else:
            for d in Data:
                if( (d['Mag']+magcorr-0.25<=Mag) and (Mag<d['Mag']+magcorr+0.25) 
                    and (d['MaxDisp']>=DriftAmpRange[0]) and (d['MaxDisp']<DriftAmpRange[1])):
                    DataOut.append(d)
                    n += 1
    except:
    ## except KeyError:
        raise SLSError('Some errors occur with %s , are you sure about the data version (Systematics/Version parameter) ?' % filename)
    if(n==0):
        raise SLSError('No systematic error parameters for magnitude Vp = %f \
        (PLATO V reference magnitude). Please turn-off systematic error or \
         change the star magnitude, the drift amplitude or use another table for the systematic errors' % Vp)
    elif(Verbose):
        print ('Systematic error parameters: number of sets of parameters matching the requirements: %i ' %n)
        
    if(n<24):
        print("WARNING ! The number of sets of systematic error parameters matching the requirements is low (<24). It is recommended to consider another drift level.")
        
    return DataOut


def GenLC(Data,seed=None,time_shift=0.,verbose=False,time_contraction=1.):
    '''

    lc,lcrn,rawlc,rawlcrn,t,Tupd,flag,LCmean,RawLCmean = GenLC(StarID,datapath,seed=None,time_shift=0.)
     
    Generate corrected and raw light-curves (LC) associated with a given star ID and given data set
     
    Inputs:
    
    StarID : the ID of the star for wich we want to generate the ligth-curve
    datapath : path to the data set
    time_shift: imposed time shift in seconds (default: 0.). Note that the CCD readouts are shifted by 6.25s
    For instance for CCD1, the time shift is 0, it is 6.25s for CCD2, 2*6.25s for CCD3 and 3*6.25 for CCD4.
    This option permits to introduce the time shift. 
    seed: assumed value of the random generator seed 
    (default: none, in that case the seed is generated on the basis of the current time)
    
    Outputs:
    lc[N]: corrected LC, N is the number of exposures
    lcrn[N]: corrected LC with random noise (RN)
    rawlc[N]: raw LC
    rawlcrn[N]: raw LC with RN 
    t[N]: exposure times
    Tupd[p]: times of the mask updates (p is the number of mask updates, the first mask used is considered as an update)
    flag[N]: flag the exposure at which a mask is updated  (or first time used)
    LCmean: mean value of the corrected LC
    RawLCmean: mean value of the raw LC
    
    '''
    

    tau0 = 90. # simulated LC duration, in days
    tau00 = tau0*86400.
    Sampling = 25.
    N = int(tau0*86400./25.)

    def simulate_segment(time,p):
        x = (time-time[0])/tau00
        return np.polyval(p,x) + 1.
    

    t = np.arange(N,dtype=np.float)*Sampling # time in seconds
    
    np.random.seed(seed)
    
    lc = np.zeros(N)
    lcrn = np.zeros(N) 

    rawlc = np.ones(N)
    rawlcvar = np.ones(N)
    rawlcrn = np.ones(N)
        
    flag = np.zeros(N,dtype=np.bool)
    
    Tupd = np.array([])


    MaskData = Data['MaskData']
    CorrectedLCData = Data['CorrectedLCData']
    if(verbose):
        SPRtot = Data['SPRtot']
        MaxDisp = Data['MaxDisp']
        print('SPRtot = %f' % SPRtot)
        print('Max drift = %f' % MaxDisp)
    RawLCData = Data['RawLCData']
    RawLCParameters =  RawLCData['parameter']
    SkyBackground = Data['SkyBackground']
    ReadOutNoise = Data['ReadOutNoise']
    Gain = Data['Gain']
    RawLCmean = RawLCData['LCmean']
    if(not (MaskData is None)):
        MaskSize = MaskData['MaskSize']

    if(MaskData is not None):
        MaskUpdNb = MaskData['MaskUpdNb']
        if(verbose):
            print('Number of mask udpates: %i' % MaskUpdNb)
        MaskUpdTime = MaskData['MaskUpTime']*time_contraction
##        print(MaskUpdNb,MaskUpdTime)
    else: # for PSF fitting method : no mask
        MaskUpdNb = 1
        MaskUpdTime = [0.]
        MaskSize = [0]
    Parameters =  CorrectedLCData['parameter']
    LCmean = CorrectedLCData['LCmean']
    # loop over the updates
    ncoef = int(Parameters.size/MaskUpdNb)
    t0 = 0.
    for i in range(MaskUpdNb):
        t1 = t0 + MaskUpdTime[i]
        if(i<MaskUpdNb-1):
            t2 = t0 + MaskUpdTime[i+1]
        else:
            t2 = t0 + tau00
                
        m = (t>=t1) & (t<t2)
        n =  m.sum()  
        if(n>0):   
            p = RawLCParameters[i*ncoef:(i+1)*ncoef]
            rawlc[m] = simulate_segment(t[m],p)*RawLCmean
            rawlcvar[m] = rawlc[m] + MaskSize[i]*(SkyBackground+ReadOutNoise**2+Gain/12.)
##            print(rawlc[m],MaskSize[i]*SkyBackground,MaskSize[i]*ReadOutNoise**2,MaskSize[i]*Gain/12.)
            rawlcrn[m] = rawlc[m] + np.random.normal(size=n)*np.sqrt(rawlcvar[m])
            p = Parameters[i*ncoef:(i+1)*ncoef]
            lc[m] = simulate_segment(t[m],p)*LCmean
            lcrn[m] = rawlcrn[m]*(lc[m]/rawlc[m])
            j = np.where(m)
            flag[j[0][0]] = True
            Tupd = np.append(Tupd,t1)
    if(time_shift>0.):
        lc = np.interp(t+time_shift,t,lc)

    return lc,lcrn,rawlc,rawlcrn,rawlcvar,t,Tupd,flag,LCmean,RawLCmean 


def SimSystematicErrorV2(Sampling,N,DataAllCamera,camera,time_shift=0.,Verbose=False,
                         seed=None,AgeSlope=0.,LCSwitchingError=0.,Normalize=True,
                         AddRandomNoise=False,TimeContractionFactor=10.):
    
    '''
    Sampling: sampling time (in seconds), must be an integer number of 25s
    N: number of exposures
    DataAllCamera: Systematic Error data associated with a given target
    camera: camera number for which the residual LC is generated
    AgeSlope: long-term slope due to the instrument aging (ppm/days)
    LCSwitchingError: LC switching error (in %)        
    '''
    Nc = len(DataAllCamera)
    if( math.fabs(Sampling/25. % 1)>1e-5  ):
        print('The sampling must be an integer number of 25s')
        raise SLSError('The sampling must be an integer number of 25s')

    nc = int(Sampling/25.)
    if(nc> 1 and Verbose):
        print('Integrating %i individual exposure(s)' % nc)
    t = np.arange(N,dtype=np.float)*Sampling # time in seconds
    
    
    lc = np.zeros(N) # normalized corrected LC
    rawlcvar = np.ones(N)  # variance of the normalized raw LC
        
    flag = np.zeros(N,dtype=np.bool)
    
    # number of quarters
    duration = N*Sampling/86400. # duration in days
    Nquarter = int(math.ceil(duration/90.))
    if(Verbose):
        print('Number of quarter: %i' % Nquarter)
    Tupd = np.array([])
    np.random.seed(seed)
    LCSwitchingErrors = np.random.normal(size=Nquarter)*(LCSwitchingError/100.)
    LCSwitchingErrors[0] = 0.
    
    seeds =  np.random.randint(0, 1073741824 + 1,size=Nquarter)
    p = int(math.floor(90.*86400.)/Sampling)
    # loop over the quarters
    for q in range(Nquarter):
        j = (q*6+camera) % Nc
        if(Verbose):
            print('Quarter: %i, Camera: %i' % (q,j))
        lc_q,lcrn_q,rawlc,rawlcrn,rawlcvar_q,t_q,Tupd_q,flag_q,LCmean,RawLCmean = GenLC(DataAllCamera[j],seed=seeds[q],time_shift=time_shift,verbose=Verbose,time_contraction=TimeContractionFactor)
        n = len(lc_q)//nc
        if(AddRandomNoise):
            lc_q = lcrn_q
        # cumulating nc exposures
        if(Normalize):
            lc_q = lc_q/LCmean 
            rawlcvar_q = rawlcvar_q/RawLCmean**2
        lc_q = rebin1d(lc_q,n)
        rawlcvar_q =  rebin1d(rawlcvar_q,n)
        flag_q = rebin1d(flag_q,n)
        t_q = rebin1d(t_q,n)/float(nc)
        
        m = min(p,N-q*p)
        t[q*p:q*p+m] = t_q[0:m] + q*p*Sampling
        s = (1. - (AgeSlope*1e-6)*(t[q*p:q*p+m]/86400.))
        lc[q*p:q*p+m] = (lc_q[0:m] + LCSwitchingErrors[q])*s
        rawlcvar[q*p:q*p+m] = rawlcvar_q[0:m]*s**2
        flag[q*p:q*p+m] = flag_q[0:m]
        Tupd = np.append(Tupd,Tupd_q)
    return lc,rawlcvar,t,Tupd,flag
        

def SimSystematicError(Sampling,N,Data,camera,seed=None,time_shift=0.,version=2,Verbose=False):

    if(version==2):
        return SimSystematicErrorV2(Sampling,N,Data,camera,time_shift=time_shift,
                         seed=seed,TimeContractionFactor=10.,Verbose=Verbose)
    def simulate_segment(time,p):
        tau0 = 90.*86400.
        x = (time-time[0])/tau0
        return np.polyval(p,x) + 1.
    
    # simulation duration, in days    
    if(version==0):
        simduration = 90. 
    else:
        simduration = 88.1 

    t = np.arange(N,dtype=np.float)*Sampling # time in seconds
    
    np.random.seed(seed)
    
    lc = np.zeros(N) # normalized corrected LC
    if(version>0):
##        rawlc = np.zeros(N) # raw LC (normalized w.r.t mean flux)
        rawlcvar = np.ones(N) # raw LC variance (normalized w.r.t mean flux)
    else:
##        rawlc = None
        rawlcvar = np.ones(N)
        
    flag = np.zeros(N,dtype=np.bool)
    
    # number of quarters
    duration = N*Sampling/86400. # duration in days
    Nquarter = int(math.ceil(duration/simduration))
    
    NData = len(Data)
    Tupd = np.array([])

    # loop over the quarters
    for q in range(Nquarter):
        j = np.random.randint(0,NData)
        try:
            if(version>0):
                MaskData = Data[j]['MaskData']
                CorrectedLCData = Data[j]['CorrectedLCData']
                RawLCData = Data[j]['RawLCData']
                RawLCParameters =  RawLCData['parameter']
                SkyBackground = Data[j]['SkyBackground']
                ReadOutNoise = Data[j]['ReadOutNoise']
                Gain = Data[j]['Gain']
                RawLCmean = RawLCData['LCmean']
                if(not (MaskData is None)):
                    MaskSize = MaskData['MaskSize']
            else:
                MaskData = Data[j]
                CorrectedLCData = Data[j]
            if(not (MaskData is None)):
                MaskUpdNb = MaskData['MaskUpdNb']
                MaskUpTime = MaskData['MaskUpTime']
            else: # for PSF fitting method : no mask
                MaskUpdNb = 1
                MaskUpTime = [0.]
                MaskSize = [36]
            Parameters =  CorrectedLCData['parameter']
        except KeyError:
            raise SLSError('Some errors occur with the systematic error data set, are you sure about the data version (Systematics/Version parameter) ?')

        # loop over the updates
        ncoef = int(Parameters.size/MaskUpdNb)
        t0 = q*simduration*86400.
        for i in range(MaskUpdNb):
            t1 = t0 + MaskUpTime[i]
            if(i<MaskUpdNb-1):
                t2 = t0 + MaskUpTime[i+1]
            else:
                t2 = t0 + simduration*86400.
                    
            m = (t>=t1) & (t<t2)   
            if(m.sum()>0):   
                if(version>0):
                    p = RawLCParameters[i*ncoef:(i+1)*ncoef]
                    rawlc = simulate_segment(t[m],p)
                    rawlcvar[m] = (rawlc + 
                                   MaskSize[i]*(SkyBackground+ReadOutNoise**2+Gain/12.)
                                   /RawLCmean)/RawLCmean*(25./Sampling)
                p = Parameters[i*ncoef:(i+1)*ncoef]
                ## p[ncoef-1] *= 3.
                ## print (p,t1,t2)
                lc[m] = simulate_segment(t[m],p)
                j = np.where(m)
                flag[j[0][0]] = True
                Tupd = np.append(Tupd,t1)
    if(time_shift>0.):
        lc = np.interp(t+time_shift,t,lc)
##        if(version>0):
##            rawlc = np.interp(t+time_shift,t,rawlc)

    return lc,rawlcvar,t,Tupd,flag


# def test_gen_up():
#     # ----------------------------------------------
#     # Simulation parameters
#     #
#     
#     # numax [muHz]
#     numax = 61.2
# 
#     # deltan_nu [muHz]
#     delta_nu  = 5.72
# 
#     # DPI [s], asymptotic period spacing
#     DPI = 232.20
# 
#     # coupling coefficient
#     q = 0.27
# 
#     # effective temperature
#     teff = 4800.
#     
#     # sampling time
#     dt = 512.
# 
#     # once generated, light-curve values are averaged over navg consecutive individual values (default: navg = 1, i.e. no average
#     # the sampling of the output light-curve will then be equal to dt*navg
#     navg = 1
# 
#     # duration in  days
#     T = 150. 
# 
#     # photon-noise reference level  [ppm.hour^(1/2)]
#     pn_ref = 7.7
# 
#     # non-photonic white-noise reference level [ppm.hour^(1/2)]
#     wn_ref = 0.
# 
#     # reference magnitude for the white-noise level 
#     mag_ref = 6.
# 
#     # star magnitude
#     mag = 12.
# 
#     # Seed of the RNG
#     seed = None
#     
#     StarID = 1
# 
#     #
#     # ----------------------------------------------
# 
#     time,ts,f,ps,mps,opar,seed = gen_up(StarID,numax,dt,T, mag , delta_nu = delta_nu, seed = seed, pn_ref = pn_ref, wn_ref = wn_ref , mag_ref = mag_ref , verbose = 1 , teff=teff, DPI = DPI , q = q , GST = 1)
# 
#         # save the light-curve
#     savelc(time,ts,StarID,opar,seed,navg=navg)
# 
#     plt.ioff()
# 
#     plt.figure(1)
#     plt.clf()
#     plt.plot(f,mps)
#     plt.xlabel(r'$\nu$ [$\mu$Hz]')
#     plt.ylabel(r'[ppm$^2$/$\mu$Hz]')
#     plt.axis(xmin=15,xmax=110)
#     plt.savefig('test_gen_up_1.png')
# 
#     plt.figure(2)
#     plt.clf()
#     plt.plot(f,ps)
#     plt.plot(f,mps,'r')
#     plt.xlabel(r'$\nu$ [$\mu$Hz]')
#     plt.ylabel(r'[ppm$^2$/$\mu$Hz]')
#     plt.loglog()
#     plt.axis(ymin=1,ymax=1e5,xmin=1e-1,xmax=300)
#     plt.savefig('test_gen_up_2.png')
# 
# 
#     plt.figure(3)
#     plt.clf()
#     plt.plot(time,ts)
#     plt.savefig('test_gen_up_3.pdf')
# 
# 
# #    plt.semilogy()
# #    plt.loglog()
# 
#     plt.show(block=False)
# 
#     s = eval(input('Type ENTER to finish'))
#     
#     return time,ts,f,ps,mps,opar
# 
# def test_gen_adipls():
#     # ----------------------------------------------
#     # Simulation parameters
#     #
#     
#     # effective temperature
#     teff = 5953.74
#     
#     # sampling time
#     dt = 32.
# 
#     # duration in  days
#     T = 150. 
# 
#     # photon-noise reference level  [ppm.hour^(1/2)]
#     pn_ref = 7.7
# 
#     # non-photonic white-noise reference level [ppm.hour^(1/2)]
#     wn_ref = 0.
# 
#     # reference magnitude for the white-noise level 
#     mag_ref = 6.
# 
#     # star magnitude
#     mag = 6.
# 
#     #  parameters a,b used for modelling surface effects
#     a,b = -0.004369 , 3.275559
#     
#     # Seed of the RNG
#     seed = None
#     
#     StarID = 2
# 
#     # Name of the .gsm generated by ADIPLS
#     fname = 'test.gsm'
# 
#     #
#     # ----------------------------------------------
#     
#     time,ts,f,ps,mps,opar,seed = gen_adipls(StarID, fname,teff,dt,T,mag,verbose=True,seed=seed,mag_ref=mag_ref,pn_ref=pn_ref,wn_ref=wn_ref, a=a,b=b,plot=True, rot_period_sur =  -1. , GST = 1 ) 
# 
#     # save the light-curve
#     savelc(time,ts,StarID,opar,seed)
# 
#     plt.ioff()
# 
# 
#     plt.figure(1)
#     plt.clf()
#     plt.plot(f,mps)
#     plt.xlabel(r'$\nu$ [$\mu$Hz]')
#     plt.ylabel(r'[ppm$^2$/$\mu$Hz]')
#     plt.axis(xmin=500,xmax=3000)
#     plt.savefig('test_gen_adipls_1.png')
# 
#     plt.figure(2)
#     plt.clf()
#     plt.plot(f,ps)
#     plt.plot(f,mps,'r')
#     plt.xlabel(r'$\nu$ [$\mu$Hz]')
#     plt.ylabel(r'[ppm$^2$/$\mu$Hz]')
#     plt.loglog()
#     plt.axis(ymin=1e-2,ymax=1e2,xmin=10,xmax=5000)
#     plt.savefig('test_gen_adipls_2.png')
# 
#     plt.figure(3)
#     plt.clf()
#     plt.plot(time,ts)
#     plt.savefig('test_gen_adipls_3.pdf')
# 
# 
# #    plt.semilogy()
# #    plt.loglog()
# 
#     plt.show(block=False)
# 
#     s = eval(input('Type ENTER to finish'))
#     
#     return time,ts,f,ps,mps,opar



## def test_single_mode():
    

##     # A :  mode amplitude  (arbitrary units)
##     A = 10.


##     #  nu0 :  mode frequency  [muHz]
##     nu0 = 3000.

##     # gamma : mode line-width  [muHz]
##     gamma = 10.

##     # white-noise (rms)
##     W = 0.0

##     # sampling time
##     dt = 50.

##     # duration in  days
##     T = 50. 

##     seed = 1999


##     #
##     gamma0=gamma*1e-6
##     nu00=nu0*1e-6 # mode frequency in Hz


##     T0 = T*86400.
##     # size of array of the time series
##     n = long(T0/(dt*2.))

##     # frequency resolution
##     dnu = 1./(2*n*dt)

##     # frequency values
##     f = (np.arange(n-1)+1)*dnu

##     # mode height
##     H = 2.*A**2/(math.pi*gamma0)

##     nyq = 1./(2.*dt) # Nyquist frequency

##     # mode profile (lorenztian profile):
##     x = 2.*(f-nu00)/gamma0
##     xp = 2.*(2.*nyq-f-nu00)/gamma0

##     # mean power spectrum
##     # the second term corresponds to the folded component
##     mpsd =   H/(1.+x**2) + H/(1.+xp**2) + 2.*W**2*dt


##     time, ts , ps =  mpsd2rts(f,mpsd,seed=seed)

##     print ("power spectrum, standard deviation: %f") % math.sqrt(np.sum(ps)*dnu)

##     print ("times series, standard deviation: %f") % np.std(ts)

##     plt.figure(0)
##     plt.clf()
##     plt.plot(time,ts)

##     plt.figure(1)
##     plt.clf()
##     plt.plot(f*1e3,ps)
##     plt.plot(f*1e3,mpsd)
##     plt.semilogy()

##     plt.show()



#!/usr/bin/env python

from __future__ import print_function
'''
from builtins import input
from builtins import str
from builtins import range
'''
import numpy as np
import math
import sls
import os
import re 
import yaml
import sys
import getopt
import shutil
import transit as tr
from scipy.stats import chi2
from packaging.version import parse as parse_version

'''

PSLS : PLATO Solar-like Light-curve Simulator
(based on SLS,  the Solar-like Light-curve Simulator, see sls.pdf and sls.py)

Copyright (c) October 2017, R. Samadi (LESIA - Observatoire de Paris)

This is a free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
 
This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this code.  If not, see <http://www.gnu.org/licenses/>.

'''


__version__ = 1.4

jupiterRadius = 71492.0  # km
ua2Km = 149.59e6  # km
NSR_Pmag = np.array([7.76,8.16,8.66,9.16,9.66,10.16,10.66,11.16,11.66,12.16,12.56,12.76,13.16,13.66,14.16,14.66,15.16,15.56])# P magnitude
NSR_Vmag = NSR_Pmag + 0.34 # V magnitude
NSR_values_24 = np.array([10.6,12.9,16.4,20.8,26.7,34.5,44.8,59.2,79.1,106.8,138.5,156.8,205.8,298.6,442.9,668.8,1018.9,1444.4])
NSR_values = NSR_values_24*math.sqrt(24.)

def VmP(teff): 
    '''
	Given the star Teff, return V-P according to Eq. 8 in  Marchiori et al (2019)
	'''
    # return -1.238e-12*teff**3 + 4.698e-8*teff**2 - 5.982e-4*teff + 2.506 # non-compliant with  Marchiori et al (2019)
    return -1.184e-12*teff**3 + 4.526e-8*teff**2 - 5.805e-4*teff + 2.449  # compliant  with  Marchiori et al (2019)

def generateZ(orbitalPeriodSecond, planetSemiMajorAxis,
                  starRadius, SamplingTime, IntegrationTime, TimeShift, sampleNumber,
                  orbitalStartAngleRad, p):
    '''
    :INPUTS:
    orbitalPeriodSecond = orbital period of the planet in second
    planetSemiMajorAxis = semi Major axis in km
    starRadius = star radius in km
    SamplingTime:int = Sampling time in second, (Plato = 25s)
    IntegrationTime : integration time in seconds
    TimeShift: time shift in seconds
    sampleNumber = number of sample we want (==> z.size)
    orbitalStartAngleRad = orbital angle in radians where to start planet position
    p = rp / r*
    :OUTPUTS:
    z = d / r*, is the normalized separation of the centers (sequence of positional offset values)

    E. Grolleau
    L.C. Smith (for the vectorized version, 13.04.2021)
    '''
    angleIncrement = SamplingTime * 2.0 * math.pi / orbitalPeriodSecond
    angle0 = orbitalStartAngleRad + (IntegrationTime/2. + TimeShift)* 2.0 * math.pi / orbitalPeriodSecond
    angles = angleIncrement * np.arange(sampleNumber) +  angle0
    # For occultquad computation we need that z < p+1
    time = np.arange(sampleNumber) * SamplingTime + IntegrationTime/2. + TimeShift
    z = np.where(
        np.sin(angles)>0,
        np.abs((planetSemiMajorAxis / starRadius) * np.cos(angles)),
        p+1
    )
    z = z.clip(min=0, max=p+1)
    return (time, z)


def psd (s,dt=1.):
    '''
 Inputs:
 s : signal (regularly sampled)
 dt: sampling (seconds)

 Outputs:
 a tuple (nu,psd)
 nu: frequencies (Hz)
 psd: power spectral density  (in Hz^-1). A double-sided PSD is assumed
    '''
    
    ft=np.fft.fft(s)
    n=len(s)
    ps=(np.abs(ft))**2*(dt/n)
    nu=np.fft.fftfreq(n,d=dt)
    nnu=n//2
    return (nu[0:nnu],ps[0:nnu])


def platotemplate(duration,dt=1.,V=11.,n=24,residual_only=False,cl=None):
    '''

 Return the total noise budget (in  ppm^2/ Hz) as a function of frequency.
 The budged includes all the random noise (including the  photon noise) and the resdiual error (after all corrections)
 It is assumed that the residual error is not correlated among the telescopes
 
 Inputs:
 duration : in days
 dt :sampling time i seconds
 V : star magnitude
 n : number of telescope (default: 24)
 
 Outputs:
 a tuple (nu,psd)
 nu: frequencies (Hz)
 psd: power spectral density  (in ppm^2 / Hz)

 cl: confidence level (<1), if specified the mean white noise level is multiplied by the threshold corresponding to the given confidence level, ,if not specified (None) the function returns the mean noise level
 '''
    V0 = 11 # reference magnitude
    scl = (24./n) # we assume that all the noises including the residual error are not correlated over the telescopes
    sclpn = 10.**( (V-V0)/2.5 ) # scaling applied on the random noise only
    if(cl!=None):
        threshold = chi2.ppf(cl,2)/chi2.mean(2) # 2 is the degree of freedom
        sclpn *= threshold
    n=int(np.ceil(86400.*duration/dt) )
    nu=np.fft.fftfreq(n,d=dt)
    m = int(n/2.)
    nu=nu[0:m]
    nu0=20e-6 # R-SCI-350, R-SCI-342 
    nu1=3e-6 #  R-SCI-350, R-SCI-342 
    s0 = 0.68 * 1e3 # R-SCI-342 
    s1 = 50.*1e3  #  R-SCI-350
    s3 = 3.0 * 1e3  # [ppm/Hz^(1/2)] random noise level at V=11 for 24 telescopes (equivalent to 50 ppm/hr)

    ps=np.zeros(m)
    j=np.where( nu >= nu0)
    if( j != -1):
        ps[j[0]] = (s0**2 + s3**2*sclpn*(residual_only==False))*scl
    j=np.where( (nu < nu0) & (nu>0.) )
    if( j != -1):
        ps[j[0]] = (np.exp( np.log(s1) + (np.log(s0)-np.log(s1)) * ((np.log(nu[j])-np.log(nu1))/(np.log(nu0)-np.log(nu1)) ))**2  + s3**2*sclpn*(residual_only==False))*scl
    return (nu,ps)
    

def pip(x,y,poly):
    '''
    test if a point is inside a polygon
    
    Taken from: http://geospatialpython.com/2011/01/point-in-polygon.html
    
    '''
    n = len(poly)
    inside = False
    p1x,p1y = poly[0]
    for i in range(n+1):
        p2x,p2y = poly[i % n]
        if y > min(p1y,p2y):
            if y <= max(p1y,p2y):
                if x <= max(p1x,p2x):
                    if p1y != p2y:
                        xints = (y-p1y)*(p2x-p1x)/(p2y-p1y)+p1x
                    if p1x == p2x or x <= xints:
                        inside = not inside
        p1x,p1y = p2x,p2y
    return inside


def rebin1d(array,n):
    nr=int(float(array.shape[0])/float(n))
    return (np.reshape(array,(n,nr))).sum(1)
 
def search_model(ModelDir,ES,logg,teff,  dlogg=0.01,dteff=15.,verbose=False,plot=False):
    
    pack = np.load(ModelDir+'data.npz')
    files = pack['files']     
    glob = pack['glob'] # star global parameters
    ## references = pack['references'] # references (&constants) parameters
    
    if(ES.lower() == 'any'):
        sel = np.ones(glob.shape[0],dtype=np.bool)
    elif(ES.lower() == 'ms'):
        sel = (glob[:,28] > 1e-3)
    elif(ES.lower() == 'sg'):
        sel = (glob[:,28] <= 1e-3) & (glob[:,20] < 200.) 
    else:
        raise sls.SLSError("unmanaged evolutionary status:"+ ES)
    
    if(sel.sum()==0):
        raise sls.SLSError("no models full fill the criteria ")
    
    glob = glob[sel,:]
    files = files[sel]
    Chi2 = ((glob[:,17]-teff)/dteff)**2 +  ((glob[:,18]-logg)/dlogg)**2
    
    i = np.argmin(Chi2)
    teffb = glob[i,17]
    loggb = glob[i,18]
    filename = files[i]
    if(type(filename) == bytes or type(filename) == np.bytes_): # solve a compatibility issue with strings coded as bytes
        filename = filename.decode()
    name = re.sub('-[n]ad\.osc','',os.path.basename(filename))
    
    if(verbose):        
        print(('Best matching, teff = %f ,logg = %f, Chi2 = %f') % (teffb, loggb,Chi2[i]))   
        print(('Star model name: %s') % (name))
        
    if(plot):
        plt.figure(200)
        plt.clf()
        plt.plot(glob[:,17],glob[:,18],'k+')
        plt.plot([teffb],[loggb],'ro')
        plt.gca().invert_yaxis()
        plt.gca().invert_xaxis()
        plt.ylabel(r'$log g$')
        plt.xlabel(r'$T_{\rm eff}$ [K]')
        plt.draw()
        
    return   name  , teffb , loggb
    
    '''
    files = pack['files']     
    glob = pack['glob'] # star global parameters
    
    
    glob[i,j],
    i: model index
    j: parameter index:
      0 : M_star
      1 : R_tot
      2 : L_tot
      3 : Z0
      4 : X0
      5 : alpha
      6 : X in CZ
      7 : Y in CZ
      8 : d2p
      9 : d2ro
      10 : age
      11 : wrot initial (global rotation velocity)
      12 : w_rot initial
      13 : g constante de la gravitaion
      14 : msun
      15 : rsun
      16 : lsol
      17 : Teff
      18 : log g
      19 : Tc temperature at the center
      20 : numax (scaling) [muHz]
      21 : deltanu (scaling) [muHz]
      22 : acoustic diameter [sec]
      23 : nuc, cutoff frequency, at the photosphere [muHz]
      24 : nuc, cutoff frequency, at the r=rmax [muHz]
      25 : deltaPI_1 [sec]
      26,27 : r1,r2 -> interval in radii on which the Brunt-Vaisala is integrated for the calculation of deltaPI
      28 : Xc
      29 : Yc
      30 : acoustic diameter [sec], computed on the basis of the .amdl file (can be oversampled)
      31 : acoustic depth of the Gamma1 bump associated with the first He ionization zone
      32 : acoustic depth of the Gamma1 bump associated with the second He ionization zone
      33 : acoustic depth of the base of the convective zone    

    references[i]: some references values:
            0: msun
            1: rsun
            2: lsun
            3: teff sun
            4: logg sun
            5: numax ref. (Mosser et al 2013)
            6: deltanu ref. (Mosser et al 2013)
            7: nuc sun (Jimenez 2006)
    '''


def usage():
    print ("usage: psls.py config.yaml")
    print ("      : ")
    print ("Options:")
    print ("-v : print program version")
    print ("-h : print this help")
    print ("-P : do some plots")
    print ("--pdf : the plots are saved as PDF otherwise as PNG (default)")
    print ("-V : verbose mode")
    print ("-f : save the LC associated with each individual camera, otherwise average over all the cameras (this is the default choice)")
    print ("-m : save the merged LC: LC from the same group of camera are averaged and then averaged LC are merged(/interlaced)")
    print ("-o <path> : output directory (the working directory is by default assumed)")
    print ("-M <number> : number of Monte-Carlo simulations performed (not yet operational)")
    print ("--extended-plots : an extended set of plots are displayed (activates automatically  the -P option)")
    



if(len(sys.argv)<2):
    usage()
    sys.exit(2)
try:
    opts,args = getopt.getopt(sys.argv[1:],"hvPVo:fmM:",["pdf","extended-plots"])

except getopt.GetoptError as err:
    print (str(err))
    usage()
    sys.exit(2)

Verbose = False
Plot = False
OutDir = '.'
FullOutput =  False  # single camera light-curves are saved 
MergedOutput = False  # LC from the same group of camera are averaged and  then averaged LC are merged(/interlaced)
Pdf = False
MC = False  # Monte-Carlo simulations on/off 
nMC = 1 # Number of  Monte-Carlo simulations
ExtendedPlots = False
for o, a in opts:
    if o == "-h" :
        usage()
        sys.exit(1)
    elif o == "-v":
        print (__version__)
        sys.exit(1)
    elif o == "-V":
        Verbose = True
    elif o == "-P":
        Plot = True
    elif o == "-f":
        FullOutput = True
    elif o == "-m":
        MergedOutput = True
    elif o == "-o":
        OutDir = a
    elif o == "-M":
        MC = True
        nMC = int(a)
    elif o == "--pdf":
            Pdf = True    
    elif o == "--extended-plots":
        ExtendedPlots = True
        Plot = True
    else:
        print ("unhandled option %s" % (o))
        sys.exit(1)


nargs = len(args)
if nargs > 1 :
    print ("too many arguments")
    usage()
    sys.exit()

if nargs < 1 :
    print ("missing arguments")
    usage()
    sys.exit()

if(MC & Plot):
    print ("The options -M and -P are not compatible. ")    
    sys.exit()

if(MC & Verbose):
    print ("The options -M and -V are not compatible. ")    
    sys.exit()

if(Plot):   
    import matplotlib
    matplotlib.use('TkAgg')
    import matplotlib.pyplot as plt
    plt.ion()

if (FullOutput & MergedOutput):
    print ("The options -m and -f are not compatible. ")
    sys.exit()
  

config=args[0]

stream = open(config, 'r')    # 'document.yaml' contains a single YAML document.
if(parse_version(yaml.__version__)< parse_version("5.0")):
    cfg = yaml.load(stream)
else:
    cfg = yaml.load(stream, Loader=yaml.FullLoader)
stream.close()

OutDir = os.path.normpath(OutDir) + '/'

Star = cfg['Star'] 
StarModelType = Star['ModelType']
StarModelName = Star['ModelName']

Osc = cfg['Oscillations']
OscEnable = Osc['Enable'] 

StarID = Star['ID']
StarName = ("%10.10i") % StarID
StarTeff,StarLogg = Star['Teff'],Star['Logg']
StarES = Star['ES']
if(Verbose):
    print ('Star name: ' + StarName)

UP = StarModelType.lower() == 'up'

if (UP):
    DPI = Osc['DPI']
    q = Osc['q']
    numax = Osc['numax']
    delta_nu =  Osc['delta_nu']    
else:
    sls.numaxref= 3050.

    if (StarModelType.lower() == 'single'):
        StarModelDir = Star['ModelDir']
    elif(StarModelType.lower() == 'grid'):
        StarModelDir = Star['ModelDir']
        StarModelDir = os.path.normpath(StarModelDir) + '/'
        if(Verbose):
            print ('requested values:')
            print (('teff = %f ,log g = %f') % (StarTeff, StarLogg))   
     
                
        StarModelName,StarTeff,StarLogg = search_model(StarModelDir,StarES,StarLogg,StarTeff,verbose=Verbose,plot=Plot)
        if(Verbose):
            print ('closest values found:')
            print (('teff = %f ,log g = %f') % (StarTeff, StarLogg))   
    else:
        print ("unhandled StarModelType: %s" % (StarModelType))
        sys.exit(1)
    if(StarModelDir is None):
        StarModelDir = './'
    StarFreqFile = StarModelDir+StarModelName + '.gsm'
    ## _, (mass, radius) = sls.read_agsm(StarFreqFile)
    ## logg = sls.logg_sun + math.log10(mass) - 2* math.log10(radius)       

    
    logTeff = math.log10(StarTeff)
    SurfaceEffects = Osc['SurfaceEffects']
    if(SurfaceEffects):
        
        if(  pip(StarTeff,StarLogg,[[5700.,4.6],[6700.,4.4],[6500.,3.9],[5700,3.9]]) == False):
            print ("surface effects: Teff and log g outside the table") 
            sys.exit(1) 

        # from Sonoi et al 2015, A&A, 583, 112  (Eq. 10 & 11)
        logma =  7.69 * logTeff -0.629 * StarLogg -28.5
        logb = -3.86*logTeff  + 0.235 * StarLogg + 14.2
    
        a =  - 10.**logma
        b = 10.**logb
        if(Verbose):
            print (('Surface effects parameters, a = %f ,b = %f') %  (a,b))
    
    else:
        a = 0.
        b = 1.


OutDir = os.path.normpath(OutDir) + '/'
Activity = cfg['Activity']
if(Activity['Enable']):
    activity = (Activity['Sigma'],Activity['Tau'])
else:
    activity = None

Granulation = cfg['Granulation']

Transit = cfg['Transit']

Observation = cfg['Observation']
Instrument = cfg['Instrument']
IntegrationTime = Instrument['IntegrationTime']

# initialization of the state of the RNG
MasterSeed = Observation['MasterSeed']
NGroup = Instrument['NGroup']
NCamera = Instrument['NCamera']  # Number of camera per group (1->6)

np.random.seed(MasterSeed)
seeds = np.random.randint(0, 1073741824 + 2,size=NGroup*NCamera+1)
Sampling,Duration,StarVMag = Instrument['Sampling'],Observation['Duration'],Star['Mag']

StarPMag = StarVMag - VmP(StarTeff)
StarVpMag = StarPMag + 0.34 # PLATO V reference magnitude (reference star of 6000K)

if(Verbose):
    print ('V magnitude: %f' % StarVMag)
    print ('P magnitude: %f' % StarPMag)
    print ('Reference V PLATO magnitude (6000 K): %f' % StarVpMag)

if (UP):
    # Simulated stellar signal, noise free (nf), UP
    time,ts,f,ps,mps_nf,opar,_ =  sls.gen_up(StarID, numax,Sampling,Duration, 
                                             StarVMag,delta_nu =  delta_nu, mass = -1. , seed =seeds[0] , pn_ref = 0.,  wn_ref= 0., mag_ref = 6.,verbose = Verbose, teff = StarTeff, DPI = DPI,  q = q , GST = 1 , incl = Star['Inclination'] , rot_f = Star['CoreRotationFreq'] , path = OutDir  , granulation = (Granulation['Enable'] == 1)  , oscillation = OscEnable)

else:
    # Simulated stellar signal, noise free
    time,ts,f,ps,mps_nf,opar,_ = sls.gen_adipls(StarID,StarFreqFile,StarTeff,Sampling,Duration,StarVMag,verbose=Verbose,seed=seeds[0],
                                            mag_ref=StarVMag,pn_ref= 0.,wn_ref= 0., a=a,b=b,plot=0, rot_period_sur =  Star['SurfaceRotationPeriod'] , 
                                            incl = Star['Inclination'] , activity = activity, granulation=Granulation['Enable'], path = OutDir , oscillation = OscEnable)


if(Transit['Enable']):
    SampleNumber = time.size
    StarRadius = opar[0]['radius']*sls.rsun*1e-5 # in km
    PlanetRadius = Transit['PlanetRadius'] * jupiterRadius  # in km
    p = PlanetRadius / StarRadius
    _, z = generateZ(Transit['OrbitalPeriod']*86400., Transit['PlanetSemiMajorAxis']*ua2Km,
              StarRadius,Sampling, IntegrationTime, 0. , SampleNumber,
              Transit['OrbitalAngle']*math.pi/180., p)
    ## gamma = [.25, .75]
    gamma = np.array(Transit['LimbDarkeningCoefficients'],dtype=np.float)
    transit = tr.occultquad(z, p, gamma, verbose=Verbose)
    if(Verbose):
        print (('Star radius [solar unit]: %f') % ( opar[0]['radius'])) 
        print ("Planet Radius/ Star Radius = {0:}".format(p))
        print (("Transit depth: %e" ) % (np.max(transit)/np.min(transit)-1.))
    if(Plot):
        plt.figure(110)
        plt.clf()
        plt.title(StarName+ ', transit')
        plt.plot(time/86400.,(transit-1.)*100.)
        plt.ylabel('Flux variation [%]')
        plt.xlabel('Time [days]')
        plt.draw()

Systematics = Instrument['Systematics']
SystematicDataVersion = int(Systematics['Version'])
DataSystematic = None
if (Systematics['Enable']):
    if(SystematicDataVersion>0):
        DriftLevel = (Systematics['DriftLevel']).lower() 
    else:
        DriftLevel = None
    DataSystematic = sls.ExtractSystematicDataMagRange(
        Systematics['Table'],StarVpMag,version=SystematicDataVersion,DriftLevel=DriftLevel,
        Verbose=Verbose,seed=seeds[-1]) 
    
# Total white-noise level ppm/Hz^(1/2), for a each single Camera
RandomNoise =  Instrument['RandomNoise']
if RandomNoise['Enable']:
    if (RandomNoise['Type'].lower() == 'user'):
        NSR = float(RandomNoise['NSR'])
    elif (RandomNoise['Type'].lower() == 'plato_scaling'):
        if( (StarPMag<NSR_Pmag[0]-0.25) or (StarPMag>NSR_Pmag[-1]+0.25) ):
            print ("Warning: Star magnitude out of range, boundary NSR value is assumed")
        NSR = np.interp(StarPMag, NSR_Pmag, NSR_values, left=NSR_values[0], right=NSR_values[-1])
    elif (RandomNoise['Type'].lower() == 'plato_simu'):
        if(DataSystematic is None):
            raise sls.SLSError("RandomNoise: when Type=PLATO_SIMU, systematic errors must also be activated")   
        if(SystematicDataVersion<1):
            raise sls.SLSError("RandomNoise: when Type=PLATO_SIMU, data version for systematic errors must be >0")   
        NSR = -1.
    else:
        raise sls.SLSError("unknown RandomNoise type: "+ RandomNoise['Type'] + ' Can be either USER, PLATO_SCALING or PLATO_SYSTEMATICS')        
else:
    NSR = 0.
W =  NSR*math.sqrt(3600.) # ppm/Hr^(1/2) -> ppm/Hz^(1/2)
opar[1]['white_noise'] = W #  ppm/Hz^(1/2)
dt = opar[1]['sampling'] 
nyq = opar[1]['nyquist']
if(Verbose and W>=0.):
        print ('NSR for one camera: %f [ppm.sqrt(hour)]' % NSR)
        print ('Total white-noise for one Camera [ppm/Hz^(1/2)]: %f' % W)
        print ('Total white-noise at sampling time [ppm]: %f' % (W/math.sqrt(Sampling)))
if(Verbose):
        print ('Nyquist frequency [muHz]: %f' % nyq)
        print ('frequency resolution [muHz]: %f' % f[0])

TimeShift = Instrument['TimeShift']
       

nt = time.size
full_ts = np.zeros((nt,NGroup,NCamera,5))
nu = np.fft.fftfreq(nt,d=dt)[0:nt//2] * 1e6 # frequencies, muHz
nnu = nu.size
spec = np.zeros((NGroup,nu.size))
dnu = nu[1]

full_ts_SC = np.zeros((nt,NGroup,NCamera)) # systematic error only

for iMC in range(nMC):
    for i in range(NGroup):
        if(Verbose):
            print (('Group: %i') % (i))
        # simulating stellar signal
        time_i, ts_i , ps_i, _ =  sls.mpsd2rts(f*1e-6,mps_nf*1e6,seed=seeds[0],time_shift=i*TimeShift)
        for j in range(NCamera): 
            full_ts[:,i,j,0] = time_i +  IntegrationTime/2.    
            ts = (1. + ts_i*1e-6  )
            tsSC = 1.
            cumul = 0.
            np.random.seed(seeds[i*NCamera+1+j])
            if(DataSystematic is not None):
                # adding systematic errors
                if(Verbose):
                    print('Generating systematic errors for camera %i of group %i' % (j,i))
                resLC,rawLCvar,_,_, flag= sls.SimSystematicError(Sampling,nt,DataSystematic,i*NCamera+j,seed=seeds[i*NCamera+1+j],version=SystematicDataVersion,Verbose=False)
                if(Verbose):
                    p2p = (np.max(resLC)/np.min(resLC)-1.)*100
                    print('      peak to peak variation [%%]: %f ' %p2p)
                    print('      number of mask updates: %i' % (np.sum(flag)-1))
                full_ts[:,i,j,4] = flag
                cumul += (resLC-1.) 
                tsSC *=  resLC
            else:
                rawLCvar = 1.
            if(W>0.):
                # adding random noise
                cumul += np.random.normal(0.,W*1e-6/math.sqrt(Sampling),size=nt) 
            if(W<0.):
                # adding random noise from LC variance
                cumul += np.random.normal(0.,1.,size=nt)*np.sqrt(rawLCvar)
            ts *= (1. + cumul)        
            if(Transit['Enable']):
                p = PlanetRadius / StarRadius
                _, z = generateZ(Transit['OrbitalPeriod']*86400., Transit['PlanetSemiMajorAxis']*ua2Km,
                                 StarRadius,Sampling, IntegrationTime, i*TimeShift , SampleNumber,
                      Transit['OrbitalAngle']*math.pi/180.,p )
                ## gamma = [.25, .75]
                ts *= tr.occultquad(z, p, gamma, verbose=Verbose)
            
            # relative flux variation, in ppm
            ts = (ts/np.mean(ts) - 1.)*1e6
            full_ts[:,i,j,1] = ts
            full_ts[:,i,j,2] = i+1 # group number 
            full_ts[:,i,j,3] = j+1 # camera number (within a given group)
            full_ts_SC[:,i,j] = (tsSC/np.mean(tsSC)-1.)*1e6
    
    # LC averaged over the camera groups 
    single_ts = np.zeros((nt,3))
    single_ts[:,0] = np.sum(full_ts[:,:,:,0],axis=(1,2))/float(NGroup*NCamera)
    single_ts[:,1] = np.sum(full_ts[:,:,:,1],axis=(1,2))/float(NGroup*NCamera)
    single_ts[:,2] = np.sum(full_ts[:,:,:,4],axis=(1,2))
    single_nu,single_psd  = psd(single_ts[:,1],dt=dt)
    single_nu *= 1e6 # Hz->muHz
    single_psd *= 1e-6 # ppm^2/Hz -> ppm^2/muHz 
    single_ts_SC = np.sum(full_ts_SC,axis=(1,2))/float(NGroup*NCamera)
    
    if(Verbose):
            print ('standard deviation of the averaged light-curves: %f ' % np.std(single_ts))

    if(MC):
        StarName = ("%7.7i%3.3i") % (StarID,iMC)
    else:
        StarName = ("%10.10i") % StarID
            
    fname = OutDir + StarName+ '.dat'
    if(Verbose):
        print ('saving the simulated light-curve as: %s' % fname)
    
    def ppar(par):
        n = len(par)
        i = 0 
        s = ''
        for u in list(par.items()):
            s += ' %s = %g' % u
            if (i < n-1):
                s += ', '
            i += 1
        return s
    
    hd = ''
    hd += ('StarID = %10.10i\n') % (StarID)  
    hd += ("Master_seed = %i\n") % (MasterSeed)
    hd += ("Version = %7.2f\n") % (__version__)
    
    
    if(FullOutput):
        full_ts = full_ts.reshape((nt*NGroup*NCamera,5))  
        np.savetxt(fname,full_ts,fmt='%12.2f %20.15e %1i %1i %1i',header=hd + '\nTime [s], Flux variation [ppm], Group ID, Camera ID, Flag')
    elif(MergedOutput):
        merged_ts = np.zeros((nt,NGroup,4))
        for G in range(NGroup):
            merged_ts[:,G,0] = rebin1d(full_ts[:,G,:,0].flatten(),nt)/float(NCamera)
            merged_ts[:,G,1] = rebin1d(full_ts[:,G,:,1].flatten(),nt)/float(NCamera)
            merged_ts[:,G,2] = G+1
            merged_ts[:,G,3] = rebin1d(full_ts[:,G,:,4].flatten(),nt)
        merged_ts = merged_ts.reshape((nt*NGroup,4)) 
        np.savetxt(fname,merged_ts,fmt='%12.2f %20.15e %1i %1i',header=hd + '\nTime [s], Flux variation [ppm], Group ID, Flag')
    else:
        np.savetxt(fname,single_ts,fmt='%12.2f %20.15e %1i',header=hd + '\nTime [s], Flux variation [ppm], Flag')
    
    hd += '# star parameters:\n'
    hd += (' teff = %f ,logg = %f\n') % (StarTeff, StarLogg)  
    hd += ppar(opar[0])
    hd += '\n# observations parameters:\n'
    hd += ppar(opar[1])
    hd += '\n# oscillation parameters:\n'
    hd += ppar(opar[2])
    hd += '\n# granulation parameters:\n'
    hd += ppar(opar[3])
    hd += '\n# activity parameters:\n'
    hd += ppar(opar[4])
    
    fname = OutDir + StarName+ '.txt'
    fd = open(fname,'w')
    fd.write(hd)
    fd.close()
    #fname = OutDir + StarName+ '.yaml'
    #shutil.copy2(config,fname)
    
    if(MC):
        MasterSeed = np.random.randint(0, 1073741824 + 1)
        np.random.seed(MasterSeed)
        seeds = np.random.randint(0, 1073741824 + 1,size=NGroup*NCamera+1)

    
if(Plot):   
    # releasing unused variables
    full_ts = None
    merged_ts = None
    if(not ExtendedPlots):
        full_ts_SC = None
    
    plt.figure(100)
    plt.clf()
    plt.title(StarName)
    

    plt.plot(single_nu[1:],single_psd[1:],'grey',label='simulated (raw)')
    win = opar[2]['numax']/100.
    m = int(round(win/single_nu[1]))
    p = int(nnu/m)
    num = rebin1d(single_nu[0:p*m],p)/float(m)
    if (DataSystematic is not None):
        _,psdsc = psd(single_ts_SC,dt=dt)
        psdsc *= 1e-6
        psdscm = rebin1d(psdsc[0:p*m],p)/float(m)
        plt.plot(single_nu[1:],psdsc[1:],'b:',label='systematics')

    psdm = rebin1d(single_psd[0:p*m],p)/float(m)
    if(W>0.):
        psdr = np.ones(mps_nf.size)*W**2*1e-6/NCamera/NGroup # random noise component
    else:
        psdr = 0.
    psdme = 0.5*mps_nf +  psdr # mean expected PSD for all camera
    
    plt.plot(num[1:],psdm[1:],'k',lw=2,label='simulated (mean)') # simulated spectrum, all camera
    if (DataSystematic is not None):
            plt.plot(num[1:],psdscm[1:],'m',label='systematics (mean)')

    # factor 1/2 to convert  the PSD from single-sided to double-sided PSD
    ## plt.plot(f[1:], psdme[1:] ,'b',lw=2,label='star+instrument')  # All Camera
    plt.plot(f[1:], 0.5*mps_nf[1:],'r',lw=2,label='star') # noise free
    if(W>0.):
        plt.plot(f[1:], psdr[1:],'g',lw=2,label='random noise') # all Camera
#    plt.plot(f[1:], 0.5*( (mps_SC[1:]  - 2*W**2*1e-6/NCamera) /(NGroup)),'m',lw=2,label='systematics') # all Camera
    
    fPT,psdPT = platotemplate(Duration, dt=1., V=11., n=NCamera*NGroup, residual_only=True)
    plt.plot(fPT[1:]*1e6, psdPT[1:]*1e-6,'k',ls='--',lw=2,label='systematics (requierements)')
    
    plt.loglog()
    plt.xlabel(r'$\nu$ [$\mu$Hz]')  
    plt.ylabel(r'[ppm$^2$/$\mu$Hz]')
    plt.axis(ymin=psdme[-1]/100.,xmax=np.max(single_nu[1:]))
    plt.legend(loc=0)
    if(Pdf):
        fname = OutDir + StarName+ '_fig1.pdf'
    else:
        fname = OutDir + StarName+ '_fig1.png'
    plt.savefig(fname)

    plt.figure(101)
    plt.clf()
    plt.title(StarName)
    plt.plot(time/86400.,single_ts[:,1]*1e-4,'grey')
        
    m = int( round(max(3600.,Sampling)/Sampling))
    p = int(time.size/m)
    tsm = rebin1d(single_ts[0:p*m,1],p)/float(m)
    timem = rebin1d(time[0:p*m],p)/float(m)
    
        
    plt.plot(timem/86400.,tsm*1e-4,'k')
    plt.xlabel('Time [days]')  
    plt.ylabel('Relative flux variation [%]')
    if(Pdf):
        fname = OutDir + StarName+ '_fig5.pdf'
    else: 
        fname = OutDir + StarName+ '_fig5.png'
    plt.savefig(fname)


    if(ExtendedPlots):

        plt.figure(102)
        plt.clf()
        plt.title(StarName)
        numax =  opar[2]['numax']
        Hmax = opar[2]['Hmax']  
        u =     (num> numax*0.5) & (num < numax*1.5)
        plt.plot(num[u],psdm[u],'k',lw=2) # simulated spectrum, all camera
    
        plt.loglog()
        plt.xlabel(r'$\nu$ [$\mu$Hz]')  
        plt.ylabel(r'[ppm$^2$/$\mu$Hz]')    
        if(Pdf):
            fname = OutDir + StarName+ '_fig2.pdf'
        else:
            fname = OutDir + StarName+ '_fig2.png'
        plt.savefig(fname)
    
    
        plt.figure(103)
        plt.clf()
        plt.title(StarName)
        u =     (f> numax*0.5) & (f < numax*1.5)
        plt.plot(f[u], 0.5*mps_nf[u],'r',lw=2,label='star') # noise free
        plt.xlabel(r'$\nu$ [$\mu$Hz]')  
        plt.ylabel(r'[ppm$^2$/$\mu$Hz]')    
        if(Pdf):
            fname = OutDir + StarName+ '_fig4.pdf'
        else: 
            fname = OutDir + StarName+ '_fig4.png'
        plt.savefig(fname)

        plt.figure(104)
        plt.clf()
        plt.title(StarName+': systematic LCs')
        for i in range(NGroup):
            for j in range(NCamera):
                plt.plot(time/86400.,full_ts_SC[:,i,j])
        plt.plot(time/86400.,single_ts_SC,'k',lw=2)
        plt.xlabel('Time [days]')  
    
    plt.draw()
    plt.show()


if(Verbose):
    print ('done')
    
if(Verbose | Plot):
    s=input('type ENTER to finish')



